@@ -1,16 +1,16 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
- * audiohandler.h
- * Copyright (C) John Stebbins 2008-2015 <stebbins@stebbins>
+ * videohandler.h
+ * Copyright (C) John Stebbins 2008-2017 <stebbins@stebbins>
  *
- * audiohandler.h is free software.
+ * videohandler.h is free software.
  *
  * You may redistribute it and/or modify it under the terms of the
  * GNU General Public License, as published by the Free Software
  * Foundation; either version 2 of the License, or (at your option)
  * any later version.
  *
- * callbacks.h is distributed in the hope that it will be useful,
+ * videohandler.h is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
@@ -25,9 +25,11 @@
 #if !defined(_VIDEOHANDLER_H_)
 #define _VIDEOHANDLER_H_
 
+#include "values.h"
 #include "settings.h"
 
-int ghb_get_video_encoder(GValue *settings);
+int ghb_get_video_encoder(GhbValue *settings);
 void ghb_video_setting_changed(GtkWidget *widget, signal_user_data_t *ud);
+void ghb_set_video_preset(GhbValue *settings, int encoder, const char * preset);
 
 #endif // _VIDEOHANDLER_H_
