@@ -1,6 +1,6 @@
 /* common.h
 
-   Copyright (c) 2003-2015 HandBrake Team
+   Copyright (c) 2003-2017 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -10,6 +10,8 @@
 #ifndef HB_COMMON_H
 #define HB_COMMON_H
 
+#include "hbtypes.h"
+#include "hb_dict.h"
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -58,6 +60,11 @@
 #ifndef MAX
 #define MAX( a, b ) ( (a) > (b) ? (a) : (b) )
 #endif
+#ifndef ABS
+#define ABS(a) ((a) > 0 ? (a) : (-(a)))
+#endif
+
+#define HB_ALIGN(x, a) (((x)+(a)-1)&~((a)-1))
 
 #ifndef HB_DEBUG_ASSERT
 #define HB_DEBUG_ASSERT(x, y) { if ((x)) { hb_error("ASSERT: %s", y); exit(1); } }
@@ -70,46 +77,19 @@
 
 #define HB_DVD_READ_BUFFER_SIZE 2048
 
-typedef struct hb_handle_s hb_handle_t;
-typedef struct hb_list_s hb_list_t;
-typedef struct hb_rate_s hb_rate_t;
-typedef struct hb_dither_s hb_dither_t;
-typedef struct hb_mixdown_s hb_mixdown_t;
-typedef struct hb_encoder_s hb_encoder_t;
-typedef struct hb_container_s hb_container_t;
-typedef struct hb_rational_s hb_rational_t;
-typedef struct hb_geometry_s hb_geometry_t;
-typedef struct hb_ui_geometry_s hb_ui_geometry_t;
-typedef struct hb_image_s hb_image_t;
-typedef struct hb_job_s  hb_job_t;
-typedef struct hb_title_set_s hb_title_set_t;
-typedef struct hb_title_s hb_title_t;
-typedef struct hb_chapter_s hb_chapter_t;
-typedef struct hb_audio_s hb_audio_t;
-typedef struct hb_audio_config_s hb_audio_config_t;
-typedef struct hb_subtitle_s hb_subtitle_t;
-typedef struct hb_subtitle_config_s hb_subtitle_config_t;
-typedef struct hb_attachment_s hb_attachment_t;
-typedef struct hb_metadata_s hb_metadata_t;
-typedef struct hb_coverart_s hb_coverart_t;
-typedef struct hb_state_s hb_state_t;
-typedef union  hb_esconfig_u     hb_esconfig_t;
-typedef struct hb_work_private_s hb_work_private_t;
-typedef struct hb_work_object_s  hb_work_object_t;
-typedef struct hb_filter_private_s hb_filter_private_t;
-typedef struct hb_filter_object_s  hb_filter_object_t;
-typedef struct hb_buffer_s hb_buffer_t;
-typedef struct hb_buffer_settings_s hb_buffer_settings_t;
-typedef struct hb_image_format_s hb_image_format_t;
-typedef struct hb_fifo_s hb_fifo_t;
-typedef struct hb_lock_s hb_lock_t;
+#define HB_MIN_WIDTH    32
+#define HB_MIN_HEIGHT   32
+#define HB_MAX_WIDTH    20480
+#define HB_MAX_HEIGHT   20480
+
 typedef enum
 {
-     HB_ERROR_NONE    = 0,
-     HB_ERROR_CANCELED   ,
-     HB_ERROR_WRONG_INPUT,
-     HB_ERROR_INIT       ,
-     HB_ERROR_UNKNOWN
+     HB_ERROR_NONE         = 0,
+     HB_ERROR_CANCELED     = 1,
+     HB_ERROR_WRONG_INPUT  = 2,
+     HB_ERROR_INIT         = 3,
+     HB_ERROR_UNKNOWN      = 4,
+     HB_ERROR_READ         = 5
 } hb_error_code;
 
 #include "ports.h"
@@ -123,9 +103,30 @@ typedef enum
 #include "libavutil/channel_layout.h"
 
 #ifdef USE_QSV
-#include "libavcodec/qsv.h"
+#include "qsv_libav.h"
 #endif
 
+struct hb_buffer_list_s
+{
+    hb_buffer_t *head;
+    hb_buffer_t *tail;
+    int count;
+    int size;
+};
+
+void hb_buffer_list_append(hb_buffer_list_t *list, hb_buffer_t *buf);
+void hb_buffer_list_prepend(hb_buffer_list_t *list, hb_buffer_t *buf);
+hb_buffer_t* hb_buffer_list_head(hb_buffer_list_t *list);
+hb_buffer_t* hb_buffer_list_rem_head(hb_buffer_list_t *list);
+hb_buffer_t* hb_buffer_list_tail(hb_buffer_list_t *list);
+hb_buffer_t* hb_buffer_list_rem_tail(hb_buffer_list_t *list);
+hb_buffer_t* hb_buffer_list_rem(hb_buffer_list_t *list, hb_buffer_t * b);
+hb_buffer_t* hb_buffer_list_clear(hb_buffer_list_t *list);
+hb_buffer_t* hb_buffer_list_set(hb_buffer_list_t *list, hb_buffer_t *buf);
+void hb_buffer_list_close(hb_buffer_list_t *list);
+int hb_buffer_list_count(hb_buffer_list_t *list);
+int hb_buffer_list_size(hb_buffer_list_t *list);
+
 hb_list_t * hb_list_init();
 int         hb_list_count( const hb_list_t * );
 void        hb_list_add( hb_list_t *, void * );
@@ -135,6 +136,7 @@ void      * hb_list_item( const hb_list_t *, int );
 void        hb_list_close( hb_list_t ** );
 
 void hb_reduce( int *x, int *y, int num, int den );
+void hb_limit_rational( int *x, int *y, int num, int den, int limit );
 void hb_reduce64( int64_t *x, int64_t *y, int64_t num, int64_t den );
 void hb_limit_rational64( int64_t *x, int64_t *y, int64_t num, int64_t den, int64_t limit );
 
@@ -156,7 +158,8 @@ void hb_audio_config_init(hb_audio_config_t * audiocfg);
 int hb_audio_add(const hb_job_t * job, const hb_audio_config_t * audiocfg);
 hb_audio_config_t * hb_list_audio_config_item(hb_list_t * list, int i);
 
-int hb_subtitle_add_ssa_header(hb_subtitle_t *subtitle, int width, int height);
+int hb_subtitle_add_ssa_header(hb_subtitle_t *subtitle, const char *font,
+                               int fs, int width, int height);
 hb_subtitle_t *hb_subtitle_copy(const hb_subtitle_t *src);
 hb_list_t *hb_subtitle_list_copy(const hb_list_t *src);
 void hb_subtitle_close( hb_subtitle_t **sub );
@@ -168,6 +171,8 @@ int hb_subtitle_can_burn( int source );
 int hb_subtitle_can_pass( int source, int mux );
 
 int hb_audio_can_apply_drc(uint32_t codec, uint32_t codec_param, int encoder);
+int hb_audio_can_apply_drc2(hb_handle_t *h, int title_idx,
+                            int audio_idx, int encoder);
 
 hb_attachment_t *hb_attachment_copy(const hb_attachment_t *src);
 
@@ -250,19 +255,16 @@ struct hb_geometry_s
     hb_rational_t par;
 };
 
-struct hb_ui_geometry_s
+struct hb_geometry_settings_s
 {
     int mode;                   // Anamorphic mode, see job struct anamorphic
     int keep;                   // Specifies settings that shouldn't be changed
     int itu_par;                // use dvd dimensions to determine PAR
     int modulus;                // pixel alignment for loose anamorphic
     int crop[4];                // Pixels cropped from source before scaling
-    int width;                  // destination storage width
-    int height;                 // destination storage height
     int maxWidth;               // max destination storage width
     int maxHeight;              // max destination storage height
-    hb_rational_t par;          // Pixel aspect used in custom anamorphic
-    hb_rational_t dar;          // Display aspect used in custom anamorphic
+    hb_geometry_t geometry;
 };
 
 struct hb_image_s
@@ -334,12 +336,21 @@ void hb_common_global_init();
 int              hb_video_framerate_get_from_name(const char *name);
 const char*      hb_video_framerate_get_name(int framerate);
 const char*      hb_video_framerate_sanitize_name(const char *name);
+void             hb_video_framerate_get_limits(int *low, int *high, int *clock);
 const hb_rate_t* hb_video_framerate_get_next(const hb_rate_t *last);
+int              hb_video_framerate_get_close(hb_rational_t *framerate,
+                                              double thresh);
 
-int              hb_audio_samplerate_get_best(uint32_t codec, int samplerate, int *sr_shift);
+int              hb_audio_samplerate_is_supported(int samplerate,
+                                                  uint32_t codec);
+int              hb_audio_samplerate_find_closest(int samplerate,
+                                                  uint32_t codec);
+int              hb_audio_samplerate_get_sr_shift(int samplerate);
 int              hb_audio_samplerate_get_from_name(const char *name);
 const char*      hb_audio_samplerate_get_name(int samplerate);
 const hb_rate_t* hb_audio_samplerate_get_next(const hb_rate_t *last);
+const hb_rate_t* hb_audio_samplerate_get_next_for_codec(const hb_rate_t *last,
+                                                        uint32_t codec);
 
 int              hb_audio_bitrate_get_best(uint32_t codec, int bitrate, int samplerate, int mixdown);
 int              hb_audio_bitrate_get_default(uint32_t codec, int samplerate, int mixdown);
@@ -349,6 +360,7 @@ const hb_rate_t* hb_audio_bitrate_get_next(const hb_rate_t *last);
 void        hb_video_quality_get_limits(uint32_t codec, float *low, float *high, float *granularity, int *direction);
 const char* hb_video_quality_get_name(uint32_t codec);
 
+int                hb_video_encoder_get_depth   (int encoder);
 const char* const* hb_video_encoder_get_presets (int encoder);
 const char* const* hb_video_encoder_get_tunes   (int encoder);
 const char* const* hb_video_encoder_get_profiles(int encoder);
@@ -376,6 +388,7 @@ int                 hb_mixdown_get_discrete_channel_count(int mixdown);
 int                 hb_mixdown_get_low_freq_channel_count(int mixdown);
 int                 hb_mixdown_get_best(uint32_t codec, uint64_t layout, int mixdown);
 int                 hb_mixdown_get_default(uint32_t codec, uint64_t layout);
+hb_mixdown_t*       hb_mixdown_get_from_mixdown(int mixdown);
 int                 hb_mixdown_get_from_name(const char *name);
 const char*         hb_mixdown_get_name(int mixdown);
 const char*         hb_mixdown_get_short_name(int mixdown);
@@ -383,6 +396,7 @@ const char*         hb_mixdown_sanitize_name(const char *name);
 const hb_mixdown_t* hb_mixdown_get_next(const hb_mixdown_t *last);
 
 int                 hb_video_encoder_get_default(int muxer);
+hb_encoder_t*       hb_video_encoder_get_from_codec(int codec);
 int                 hb_video_encoder_get_from_name(const char *name);
 const char*         hb_video_encoder_get_name(int encoder);
 const char*         hb_video_encoder_get_short_name(int encoder);
@@ -399,6 +413,7 @@ const hb_encoder_t* hb_video_encoder_get_next(const hb_encoder_t *last);
  */
 int                 hb_audio_encoder_get_fallback_for_passthru(int passthru);
 int                 hb_audio_encoder_get_default(int muxer);
+hb_encoder_t*       hb_audio_encoder_get_from_codec(int codec);
 int                 hb_audio_encoder_get_from_name(const char *name);
 const char*         hb_audio_encoder_get_name(int encoder);
 const char*         hb_audio_encoder_get_short_name(int encoder);
@@ -414,6 +429,7 @@ void hb_autopassthru_apply_settings(hb_job_t *job);
 void hb_autopassthru_print_settings(hb_job_t *job);
 int  hb_autopassthru_get_encoder(int in_codec, int copy_mask, int fallback, int muxer);
 
+hb_container_t*       hb_container_get_from_format(int format);
 int                   hb_container_get_from_name(const char *name);
 int                   hb_container_get_from_extension(const char *extension); // not really a container name
 const char*           hb_container_get_name(int format);
@@ -428,16 +444,16 @@ struct hb_title_set_s
 {
     hb_list_t   * list_title;
     int           feature;    // Detected DVD feature title
+    char          path[1024];
 };
 
-extern int hb_gui_use_hwd_flag;
-
 typedef enum
 {
     HB_ANAMORPHIC_NONE,
     HB_ANAMORPHIC_STRICT,
     HB_ANAMORPHIC_LOOSE,
-    HB_ANAMORPHIC_CUSTOM
+    HB_ANAMORPHIC_CUSTOM,
+    HB_ANAMORPHIC_AUTO
 } hb_anamorphic_mode_t;
 
 /******************************************************************************
@@ -446,6 +462,8 @@ typedef enum
  *****************************************************************************/
 struct hb_job_s
 {
+    PRIVATE const char  * json;   // JSON encoded job string
+
     /* ID assigned by UI so it can groups job passes together */
     int             sequence_id;
 
@@ -460,67 +478,61 @@ struct hb_job_s
     /* Include chapter marker track in mp4? */
     int             chapter_markers;
 
-    /* Picture settings:
-         crop:                must be multiples of 2 (top/bottom/left/right)
-         deinterlace:         0 or 1
-         width:               must be a multiple of 2
-         height:              must be a multiple of 2
-         grayscale:           black and white encoding
-         pixel_ratio:         store pixel aspect ratio in the video
-         pixel_aspect_width:  numerator for pixel aspect ratio
-         pixel_aspect_height: denominator for pixel aspect ratio
-         modulus:             set a number for dimensions to be multiples of
-         maxWidth:            keep width below this
-         maxHeight:           keep height below this */
-    int             crop[4];
-    int             deinterlace;
+    // Video filters
+    int             grayscale;      // Black and white encoding
     hb_list_t     * list_filter;
-    int             width;
-    int             height;
-    int             grayscale;
 
-    struct
-    {
-        hb_anamorphic_mode_t  mode;
-        int                   itu_par;
-        int                   par_width;
-        int                   par_height;
-        int                   dar_width;  // 0 if normal
-        int                   dar_height; // 0 if normal
-        int                   keep_display_aspect;
-    } anamorphic;
-
-    int             modulus;
-    int             maxWidth;
-    int             maxHeight;
+    PRIVATE int             crop[4];
+    PRIVATE int             width;
+    PRIVATE int             height;
+    hb_rational_t           par;
 
     /* Video settings:
          vcodec:            output codec
-         vquality:          output quality (if < 0.0, bitrate is used instead)
+         vquality:          output quality (if invalid, bitrate is used instead)
          vbitrate:          output bitrate (Kbps)
-         vrate, vrate_base: output framerate is vrate / vrate_base
+         vrate:             output framerate
          cfr:               0 (vfr), 1 (cfr), 2 (pfr) [see render.c]
          pass:              0, 1 or 2 (or -1 for scan)
          areBframes:        boolean to note if b-frames are used */
-#define HB_VCODEC_MASK         0x0000FFF
-#define HB_VCODEC_X264         0x0000001
+#define HB_VCODEC_MASK         0x00FFFFF
+#define HB_VCODEC_INVALID      0x0000000
 #define HB_VCODEC_THEORA       0x0000002
-#define HB_VCODEC_X265         0x0000004
 #define HB_VCODEC_FFMPEG_MPEG4 0x0000010
 #define HB_VCODEC_FFMPEG_MPEG2 0x0000020
 #define HB_VCODEC_FFMPEG_VP8   0x0000040
+#define HB_VCODEC_FFMPEG_VP9   0x0000080
 #define HB_VCODEC_FFMPEG_MASK  0x00000F0
 #define HB_VCODEC_QSV_H264     0x0000100
+#define HB_VCODEC_QSV_H265     0x0000200
 #define HB_VCODEC_QSV_MASK     0x0000F00
-#define HB_VCODEC_H264_MASK    (HB_VCODEC_X264|HB_VCODEC_QSV_H264)
+#define HB_VCODEC_X264_8BIT    0x0010000
+#define HB_VCODEC_X264         HB_VCODEC_X264_8BIT
+#define HB_VCODEC_X264_10BIT   0x0020000
+#define HB_VCODEC_X264_MASK    0x0030000
+#define HB_VCODEC_H264_MASK    (HB_VCODEC_X264_MASK|HB_VCODEC_QSV_H264)
+#define HB_VCODEC_X265_8BIT    0x0001000
+#define HB_VCODEC_X265         HB_VCODEC_X265_8BIT
+#define HB_VCODEC_X265_10BIT   0x0002000
+#define HB_VCODEC_X265_12BIT   0x0004000
+#define HB_VCODEC_X265_16BIT   0x0008000
+#define HB_VCODEC_X265_MASK    0x000F000
+#define HB_VCODEC_H265_MASK    (HB_VCODEC_X265_MASK|HB_VCODEC_QSV_H265)
+
+/* define an invalid CQ value compatible with all CQ-capable codecs */
+#define HB_INVALID_VIDEO_QUALITY (-1000.)
 
     int             vcodec;
-    float           vquality;
+    double          vquality;
     int             vbitrate;
-    int             vrate;
-    int             vrate_base;
+    hb_rational_t   vrate;
+    // Some parameters that depend on vrate (like keyint) can't change
+    // between encoding passes. So orig_vrate is used to store the
+    // 1st pass rate.
+    hb_rational_t   orig_vrate;
     int             cfr;
-    int             pass;
+    PRIVATE int     pass_id;
+    int             twopass;        // Enable 2-pass encode. Boolean
     int             fastfirstpass;
     char           *encoder_preset;
     char           *encoder_tune;
@@ -533,21 +545,29 @@ struct hb_job_s
     int             color_prim;
     int             color_transfer;
     int             color_matrix;
-// see https://developer.apple.com/quicktime/icefloe/dispatch019.html#colr
-#define HB_COLR_PRI_BT709     1
-#define HB_COLR_PRI_UNDEF     2
-#define HB_COLR_PRI_EBUTECH   5 // use for bt470bg
-#define HB_COLR_PRI_SMPTEC    6 // smpte170m; also use for bt470m and smpte240m
-// 0, 3-4, 7-65535: reserved
-#define HB_COLR_TRA_BT709     1 // also use for bt470m, bt470bg and smpte170m
-#define HB_COLR_TRA_UNDEF     2
-#define HB_COLR_TRA_SMPTE240M 7
-// 0, 3-6, 8-65535: reserved
-#define HB_COLR_MAT_BT709     1
-#define HB_COLR_MAT_UNDEF     2
-#define HB_COLR_MAT_SMPTE170M 6 // also use for fcc and bt470bg
-#define HB_COLR_MAT_SMPTE240M 7
-// 0, 3-5, 8-65535: reserved
+// see https://developer.apple.com/library/content/technotes/tn2162/_index.html
+//     https://developer.apple.com/library/content/documentation/QuickTime/QTFF/QTFFChap3/qtff3.html#//apple_ref/doc/uid/TP40000939-CH205-125526
+//     libav pixfmt.h
+#define HB_COLR_PRI_BT709        1
+#define HB_COLR_PRI_UNDEF        2
+#define HB_COLR_PRI_EBUTECH      5 // use for bt470bg
+#define HB_COLR_PRI_SMPTEC       6 // smpte170m; also use for bt470m and smpte240m
+#define HB_COLR_PRI_BT2020       9
+// 0, 3-4, 7-8, 10-65535: reserved/not implemented
+#define HB_COLR_TRA_BT709        1 // also use for bt470m, bt470bg, smpte170m, bt2020_10 and bt2020_12
+#define HB_COLR_TRA_UNDEF        2
+#define HB_COLR_TRA_SMPTE240M    7
+#define HB_COLR_TRA_BT2020_10    14
+#define HB_COLR_TRA_BT2020_12    15
+#define HB_COLR_TRA_SMPTEST2084  16
+// 0, 3-6, 8-15, 17-65535: reserved/not implemented
+#define HB_COLR_MAT_BT709        1
+#define HB_COLR_MAT_UNDEF        2
+#define HB_COLR_MAT_SMPTE170M    6 // also use for fcc and bt470bg
+#define HB_COLR_MAT_SMPTE240M    7
+#define HB_COLR_MAT_BT2020_NCL   9
+#define HB_COLR_MAT_BT2020_CL    10
+// 0, 3-5, 8, 11-65535: reserved/not implemented
 
     hb_list_t     * list_chapter;
 
@@ -569,6 +589,7 @@ struct hb_job_s
      *     file: file path
      */
 #define HB_MUX_MASK     0xFF0001
+#define HB_MUX_INVALID  0x000000
 #define HB_MUX_MP4V2    0x010000
 #define HB_MUX_AV_MP4   0x020000
 #define HB_MUX_MASK_MP4 0x030000
@@ -583,8 +604,6 @@ struct hb_job_s
     int             mux;
     char          * file;
 
-    /* Allow MP4 files > 4 gigs */
-    int             largeFileSize;
     int             mp4_optimize;
     int             ipod_atom;
 
@@ -605,17 +624,17 @@ struct hb_job_s
                                         //  initially (for frame accurate positioning
                                         //  to non-I frames).
     int use_opencl;
-    int use_hwd;
-    int use_decomb;
-    int use_detelecine;
+    PRIVATE int use_decomb;
+    PRIVATE int use_detelecine;
 
-#ifdef USE_QSV
     // QSV-specific settings
     struct
     {
         int decode;
         int async_depth;
-        av_qsv_context *ctx;
+#ifdef USE_QSV
+        hb_qsv_context *ctx;
+#endif
         // shared encoding parameters
         // initialized by the QSV encoder, then used upstream (e.g. by filters)
         // to configure their output so that it matches what the encoder expects
@@ -627,12 +646,10 @@ struct hb_job_s
             int is_init_done;
         } enc_info;
     } qsv;
-#endif
 
 #ifdef __LIBHB__
     /* Internal data */
     hb_handle_t   * h;
-    hb_lock_t     * pause;
     volatile hb_error_code * done_error;
     volatile int  * die;
     volatile int    done;
@@ -656,7 +673,8 @@ struct hb_job_s
 
 /* Audio starts here */
 /* Audio Codecs: Update win/CS/HandBrake.Interop/HandBrakeInterop/HbLib/NativeConstants.cs when changing these consts */
-#define HB_ACODEC_MASK      0x00FFFF00
+#define HB_ACODEC_INVALID   0x00000000
+#define HB_ACODEC_MASK      0x07FFFF00
 #define HB_ACODEC_LAME      0x00000200
 #define HB_ACODEC_VORBIS    0x00000400
 #define HB_ACODEC_AC3       0x00000800
@@ -672,16 +690,22 @@ struct hb_job_s
 #define HB_ACODEC_FFFLAC24  0x00200000
 #define HB_ACODEC_FDK_AAC   0x00400000
 #define HB_ACODEC_FDK_HAAC  0x00800000
-#define HB_ACODEC_FF_MASK   0x00FF2800
+#define HB_ACODEC_FFEAC3    0x01000000
+#define HB_ACODEC_FFTRUEHD  0x02000000
+#define HB_ACODEC_OPUS      0x04000000
+#define HB_ACODEC_FF_MASK   0x07FF2800
 #define HB_ACODEC_PASS_FLAG 0x40000000
-#define HB_ACODEC_PASS_MASK (HB_ACODEC_MP3 | HB_ACODEC_FFAAC | HB_ACODEC_DCA_HD | HB_ACODEC_AC3 | HB_ACODEC_DCA)
-#define HB_ACODEC_AUTO_PASS (HB_ACODEC_PASS_MASK | HB_ACODEC_PASS_FLAG)
-#define HB_ACODEC_MP3_PASS  (HB_ACODEC_MP3 | HB_ACODEC_PASS_FLAG)
-#define HB_ACODEC_AAC_PASS  (HB_ACODEC_FFAAC | HB_ACODEC_PASS_FLAG)
-#define HB_ACODEC_AC3_PASS  (HB_ACODEC_AC3 | HB_ACODEC_PASS_FLAG)
-#define HB_ACODEC_DCA_PASS  (HB_ACODEC_DCA | HB_ACODEC_PASS_FLAG)
-#define HB_ACODEC_DCA_HD_PASS  (HB_ACODEC_DCA_HD | HB_ACODEC_PASS_FLAG)
-#define HB_ACODEC_ANY       (HB_ACODEC_MASK | HB_ACODEC_PASS_FLAG)
+#define HB_ACODEC_PASS_MASK   (HB_ACODEC_AC3 | HB_ACODEC_DCA | HB_ACODEC_DCA_HD | HB_ACODEC_FFAAC | HB_ACODEC_FFEAC3 | HB_ACODEC_FFFLAC | HB_ACODEC_MP3 | HB_ACODEC_FFTRUEHD)
+#define HB_ACODEC_AUTO_PASS   (HB_ACODEC_PASS_FLAG | HB_ACODEC_PASS_MASK)
+#define HB_ACODEC_ANY         (HB_ACODEC_PASS_FLAG | HB_ACODEC_MASK)
+#define HB_ACODEC_AAC_PASS    (HB_ACODEC_PASS_FLAG | HB_ACODEC_FFAAC)
+#define HB_ACODEC_AC3_PASS    (HB_ACODEC_PASS_FLAG | HB_ACODEC_AC3)
+#define HB_ACODEC_DCA_PASS    (HB_ACODEC_PASS_FLAG | HB_ACODEC_DCA)
+#define HB_ACODEC_DCA_HD_PASS (HB_ACODEC_PASS_FLAG | HB_ACODEC_DCA_HD)
+#define HB_ACODEC_EAC3_PASS   (HB_ACODEC_PASS_FLAG | HB_ACODEC_FFEAC3)
+#define HB_ACODEC_FLAC_PASS   (HB_ACODEC_PASS_FLAG | HB_ACODEC_FFFLAC)
+#define HB_ACODEC_MP3_PASS    (HB_ACODEC_PASS_FLAG | HB_ACODEC_MP3)
+#define HB_ACODEC_TRUEHD_PASS (HB_ACODEC_PASS_FLAG | HB_ACODEC_FFTRUEHD)
 
 #define HB_SUBSTREAM_BD_TRUEHD  0x72
 #define HB_SUBSTREAM_BD_AC3     0x76
@@ -718,8 +742,8 @@ struct hb_audio_config_s
         int      samplerate; /* Output sample rate (Hz) */
         int      samples_per_frame; /* Number of samples per frame */
         int      bitrate; /* Output bitrate (Kbps) */
-        float    quality; /* Output quality (encoder-specific) */
-        float    compression_level;  /* Output compression level (encoder-specific) */
+        double   quality; /* Output quality (encoder-specific) */
+        double   compression_level;  /* Output compression level (encoder-specific) */
         double   dynamic_range_compression; /* Amount of DRC applied to this track */
         double   gain; /* Gain (in dB), negative is quieter */
         int      normalize_mix_level; /* mix level normalization (boolean) */
@@ -741,6 +765,7 @@ struct hb_audio_config_s
         PRIVATE uint32_t flags; /* Bitstream flags, codec-specific */
         PRIVATE uint32_t mode; /* Bitstream mode, codec-specific */
         PRIVATE int samplerate; /* Input sample rate (Hz) */
+        PRIVATE int sample_bit_depth; /* Input samples' bit depth (0 if unknown) */
         PRIVATE int samples_per_frame; /* Number of samples per frame */
         PRIVATE int bitrate; /* Input bitrate (bps) */
         PRIVATE int matrix_encoding; /* Source matrix encoding mode, set by the audio decoder */
@@ -753,6 +778,12 @@ struct hb_audio_config_s
         PRIVATE char description[1024];
         PRIVATE char simple[1024];
         PRIVATE char iso639_2[4];
+#define HB_AUDIO_TYPE_NONE              0
+#define HB_AUDIO_TYPE_NORMAL            1
+#define HB_AUDIO_TYPE_VISUALLY_IMPAIRED 2
+#define HB_AUDIO_TYPE_COMMENTARY        3
+#define HB_AUDIO_TYPE_ALT_COMMENTARY    4
+#define HB_AUDIO_TYPE_BD_SECONDARY      5
         PRIVATE uint8_t type; /* normal, visually impaired, director's commentary */
     } lang;
 };
@@ -869,7 +900,6 @@ struct hb_subtitle_s
 
     hb_fifo_t * fifo_in;  /* SPU ES */
     hb_fifo_t * fifo_raw; /* Decoded SPU */
-    hb_fifo_t * fifo_sync;/* Synced */
     hb_fifo_t * fifo_out; /* Correct Timestamps, ready to be muxed */
     hb_mux_data_t * mux_data;
 #endif
@@ -882,7 +912,7 @@ struct hb_subtitle_s
  */
 struct hb_attachment_s
 {
-    enum attachtype { FONT_TTF_ATTACH, HB_ART_ATTACH } type;
+    enum attachtype { FONT_TTF_ATTACH, FONT_OTF_ATTACH, HB_ART_ATTACH } type;
     char *  name;
     char *  data;
     int     size;
@@ -920,79 +950,70 @@ struct hb_metadata_s
 struct hb_title_s
 {
     enum { HB_DVD_TYPE, HB_BD_TYPE, HB_STREAM_TYPE, HB_FF_STREAM_TYPE } type;
-    uint32_t    reg_desc;
-    char        path[1024];
-    char        name[1024];
-    int         index;
-    int         playlist;
-    int         vts;
-    int         ttn;
-    int         cell_start;
-    int         cell_end;
-    uint64_t    block_start;
-    uint64_t    block_end;
-    uint64_t    block_count;
-    int         angle_count;
-    void        *opaque_priv;
+    uint32_t      reg_desc;
+    char          path[1024];
+    char          name[1024];
+    int           index;
+    int           playlist;
+    int           vts;
+    int           ttn;
+    int           cell_start;
+    int           cell_end;
+    uint64_t      block_start;
+    uint64_t      block_end;
+    uint64_t      block_count;
+    int           angle_count;
+    void        * opaque_priv;
 
     /* Visual-friendly duration */
-    int         hours;
-    int         minutes;
-    int         seconds;
+    int           hours;
+    int           minutes;
+    int           seconds;
 
     /* Exact duration (in 1/90000s) */
-    uint64_t    duration;
+    uint64_t      duration;
 
-    double      aspect;             // aspect ratio for the title's video
-    double      container_aspect;   // aspect ratio from container (0 if none)
-    int         has_resolution_change;
-    int         width;
-    int         height;
-    int         pixel_aspect_width;
-    int         pixel_aspect_height;
-    int         color_prim;
-    int         color_transfer;
-    int         color_matrix;
-    int         rate;
-    int         rate_base;
-    int         crop[4];
+    int           preview_count;
+    int           has_resolution_change;
+    hb_geometry_t geometry;
+    hb_rational_t dar;             // aspect ratio for the title's video
+    hb_rational_t container_dar;   // aspect ratio from container (0 if none)
+    int           color_prim;
+    int           color_transfer;
+    int           color_matrix;
+    hb_rational_t vrate;
+    int           crop[4];
     enum {HB_DVD_DEMUXER, HB_TS_DEMUXER, HB_PS_DEMUXER, HB_NULL_DEMUXER} demuxer;
-    int         detected_interlacing;
-    int         pcr_pid;                /* PCR PID for TS streams */
-    int         video_id;               /* demuxer stream id for video */
-    int         video_codec;            /* worker object id of video codec */
-    uint32_t    video_stream_type;      /* stream type from source stream */
-    int         video_codec_param;      /* codec specific config */
-    char        *video_codec_name;
-    int         video_bitrate;
-    char        *container_name;
-    int         data_rate;
+    int           detected_interlacing;
+    int           pcr_pid;                /* PCR PID for TS streams */
+    int           video_id;               /* demuxer stream id for video */
+    int           video_codec;            /* worker object id of video codec */
+    uint32_t      video_stream_type;      /* stream type from source stream */
+    int           video_codec_param;      /* codec specific config */
+    char        * video_codec_name;
+    int           video_bitrate;
+    char        * container_name;
+    int           data_rate;
 
     // additional supported video decoders (e.g. HW-accelerated implementations)
-    int video_decode_support;
-#define HB_DECODE_SUPPORT_SW  0x01 // software (libavcodec or mpeg2dec)
-#define HB_DECODE_SUPPORT_QSV 0x02 // Intel Quick Sync Video
+    int           video_decode_support;
+#define HB_DECODE_SUPPORT_SW    0x01 // software (libavcodec or mpeg2dec)
+#define HB_DECODE_SUPPORT_QSV   0x02 // Intel Quick Sync Video
 
-    hb_metadata_t *metadata;
+    hb_metadata_t * metadata;
 
-    hb_list_t * list_chapter;
-    hb_list_t * list_audio;
-    hb_list_t * list_subtitle;
-    hb_list_t * list_attachment;
+    hb_list_t     * list_chapter;
+    hb_list_t     * list_audio;
+    hb_list_t     * list_subtitle;
+    hb_list_t     * list_attachment;
 
-#define HB_TITLE_JOBS
-#if defined(HB_TITLE_JOBS)
-    hb_job_t  * job;
-#endif
-
-    uint32_t    flags;
+    uint32_t        flags;
                 // set if video stream doesn't have IDR frames
 #define         HBTF_NO_IDR (1 << 0)
 #define         HBTF_SCAN_COMPLETE (1 << 1)
 
     // whether OpenCL scaling is supported for this source
-    int opencl_support;
-    int hwd_support; // TODO: merge to video_decode_support
+    int             opencl_support;
 };
 
 // Update win/CS/HandBrake.Interop/HandBrakeInterop/HbLib/hb_state_s.cs when changing this struct
@@ -1023,9 +1044,14 @@ struct hb_state_s
         struct
         {
             /* HB_STATE_WORKING */
+#define HB_PASS_SUBTITLE    -1
+#define HB_PASS_ENCODE      0
+#define HB_PASS_ENCODE_1ST  1   // Some code depends on these values being
+#define HB_PASS_ENCODE_2ND  2   // 1 and 2.  Do not change.
+            int   pass_id;
+            int   pass;
+            int   pass_count;
             float progress;
-            int   job_cur;
-            int   job_count;
             float rate_cur;
             float rate_avg;
             int   hours;
@@ -1050,33 +1076,30 @@ struct hb_state_s
 
 typedef struct hb_work_info_s
 {
-    const char * name;
-    int          profile;
-    int          level;
-    int          bitrate;
-    int          rate;
-    int          rate_base;
-    uint32_t     version;
-    uint32_t     flags;
-    uint32_t     mode;
+    const char  * name;
+    int           profile;
+    int           level;
+    int           bitrate;
+    hb_rational_t rate;
+    uint32_t      version;
+    uint32_t      flags;
+    uint32_t      mode;
     union
     {
         struct
         {    // info only valid for video decoders
-            int width;
-            int height;
-            int pixel_aspect_width;
-            int pixel_aspect_height;
-            int color_prim;
-            int color_transfer;
-            int color_matrix;
-            int video_decode_support;
+            hb_geometry_t geometry;
+            int           color_prim;
+            int           color_transfer;
+            int           color_matrix;
+            int           video_decode_support;
         };
         struct
         {    // info only valid for audio decoders
             uint64_t channel_layout;
             hb_chan_map_t * channel_map;
             int samples_per_frame;
+            int sample_bit_depth;
             int matrix_encoding;
         };
     };
@@ -1120,17 +1143,20 @@ struct hb_work_object_s
 
     hb_thread_t       * thread;
     volatile int      * done;
+    volatile int      * die;
     int                 status;
     int                 codec_param;
     hb_title_t        * title;
 
     hb_work_object_t  * next;
-    int                 thread_sleep_interval;
+
+    hb_handle_t       * h;
 #endif
 };
 
 extern hb_work_object_t hb_sync_video;
 extern hb_work_object_t hb_sync_audio;
+extern hb_work_object_t hb_sync_subtitle;
 extern hb_work_object_t hb_decvobsub;
 extern hb_work_object_t hb_encvobsub;
 extern hb_work_object_t hb_deccc608;
@@ -1147,7 +1173,6 @@ extern hb_work_object_t hb_encx265;
 extern hb_work_object_t hb_decavcodeca;
 extern hb_work_object_t hb_decavcodecv;
 extern hb_work_object_t hb_declpcm;
-extern hb_work_object_t hb_enclame;
 extern hb_work_object_t hb_encvorbis;
 extern hb_work_object_t hb_muxer;
 extern hb_work_object_t hb_encca_aac;
@@ -1161,71 +1186,69 @@ extern hb_work_object_t hb_reader;
 #define HB_FILTER_DROP    3
 #define HB_FILTER_DONE    4
 
-int hb_use_dxva(hb_title_t *title);
-
 typedef struct hb_filter_init_s
 {
-    hb_job_t    * job;
-    int           pix_fmt;
-    int           width;
-    int           height;
-    int           par_width;
-    int           par_height;
-    int           crop[4];
-    int           vrate_base;
-    int           vrate;
-    int           cfr;
-    int           use_dxva;
+    hb_job_t      * job;
+    int             pix_fmt;
+    hb_geometry_t   geometry;
+    int             crop[4];
+    hb_rational_t   vrate;
+    int             cfr;
+    int             grayscale;
 } hb_filter_init_t;
 
 typedef struct hb_filter_info_s
 {
-    char               human_readable_desc[128];
+    char             * human_readable_desc;
     hb_filter_init_t   out;
 } hb_filter_info_t;
 
 struct hb_filter_object_s
 {
-    int                     id;
-    int                     enforce_order;
-    char                  * name;
-    char                  * settings;
+    int                   id;
+    int                   enforce_order;
+    char                * name;
+    hb_dict_t           * settings;
 
 #ifdef __LIBHB__
-    int         (* init)  ( hb_filter_object_t *, hb_filter_init_t * );
+    int                (* init)     ( hb_filter_object_t *, hb_filter_init_t * );
+    int                (* post_init)( hb_filter_object_t *, hb_job_t * );
+    int                (* work)     ( hb_filter_object_t *,
+                                      hb_buffer_t **, hb_buffer_t ** );
+    void               (* close)    ( hb_filter_object_t * );
+    hb_filter_info_t * (* info)     ( hb_filter_object_t * );
 
-    int         (* work)  ( hb_filter_object_t *,
-                            hb_buffer_t **, hb_buffer_t ** );
+    const char          * settings_template;
 
-    void        (* close) ( hb_filter_object_t * );
-    int         (* info)  ( hb_filter_object_t *, hb_filter_info_t * );
+    hb_fifo_t           * fifo_in;
+    hb_fifo_t           * fifo_out;
 
-    hb_fifo_t   * fifo_in;
-    hb_fifo_t   * fifo_out;
-
-    hb_subtitle_t     * subtitle;
+    hb_subtitle_t       * subtitle;
 
     hb_filter_private_t * private_data;
 
-    hb_thread_t       * thread;
-    volatile int      * done;
-    int                 status;
+    hb_thread_t         * thread;
+    volatile int        * done;
+    int                   status;
 
     // Filters can drop frames and thus chapter marks
     // These are used to bridge the chapter to the next buffer
-    int                 chapter_val;
-    int64_t             chapter_time;
+    int                   chapter_val;
+    int64_t               chapter_time;
 #endif
 };
 
 // Update win/CS/HandBrake.Interop/HandBrakeInterop/HbLib/hb_filter_ids.cs when changing this enum
 enum
 {
+    HB_FILTER_INVALID = 0,
     // for QSV - important to have before other filters
+    HB_FILTER_FIRST = 1,
     HB_FILTER_QSV_PRE = 1,
 
     // First, filters that may change the framerate (drop or dup frames)
     HB_FILTER_DETELECINE,
+    HB_FILTER_COMB_DETECT,
     HB_FILTER_DECOMB,
     HB_FILTER_DEINTERLACE,
     HB_FILTER_VFR,
@@ -1236,54 +1259,60 @@ enum
     HB_FILTER_NLMEANS,
     HB_FILTER_RENDER_SUB,
     HB_FILTER_CROP_SCALE,
+    HB_FILTER_ROTATE,
+    HB_FILTER_GRAYSCALE,
+    HB_FILTER_PAD,
 
     // Finally filters that don't care what order they are in,
     // except that they must be after the above filters
-    HB_FILTER_ROTATE,
+    HB_FILTER_AVFILTER,
 
     // for QSV - important to have as a last one
     HB_FILTER_QSV_POST,
     // default MSDK VPP filter
     HB_FILTER_QSV,
+    HB_FILTER_LAST = HB_FILTER_QSV
 };
 
+hb_filter_object_t * hb_filter_get( int filter_id );
 hb_filter_object_t * hb_filter_init( int filter_id );
 hb_filter_object_t * hb_filter_copy( hb_filter_object_t * filter );
-hb_list_t *hb_filter_list_copy(const hb_list_t *src);
-void hb_filter_close( hb_filter_object_t ** );
-char * hb_generate_filter_settings(int filter_id, const char *preset,
-                                                  const char *tune);
-int    hb_validate_filter_settings(int filter_id, const char *filter_param);
-int    hb_validate_param_string(const char *regex_pattern, const char *param_string);
+hb_list_t          * hb_filter_list_copy(const hb_list_t *src);
+hb_filter_object_t * hb_filter_find(const hb_list_t *list, int filter_id);
+void                 hb_filter_close( hb_filter_object_t ** );
+void                 hb_filter_info_close( hb_filter_info_t ** );
+hb_dict_t          * hb_parse_filter_settings(const char * settings);
+char               * hb_parse_filter_settings_json(const char * settings_str);
+char               * hb_filter_settings_string(int filter_id,
+                                               hb_value_t * value);
+char               * hb_filter_settings_string_json(int filter_id,
+                                                    const char * json);
 
 typedef void hb_error_handler_t( const char *errmsg );
 
 extern void hb_register_error_handler( hb_error_handler_t * handler );
 
+char * hb_strdup_vaprintf( const char * fmt, va_list args );
 char * hb_strdup_printf(const char *fmt, ...) HB_WPRINTF(1, 2);
 char * hb_strncat_dup( const char * s1, const char * s2, size_t n );
 
+// free array of strings
+void    hb_str_vfree( char ** strv );
+// count number of strings in array of strings
+int     hb_str_vlen(char ** strv);
+// split string into array of strings
+char ** hb_str_vsplit( const char * str, char delem );
+
 int hb_yuv2rgb(int yuv);
 int hb_rgb2yuv(int rgb);
 
 const char * hb_subsource_name( int source );
 
 // unparse a set of x264 settings to an HB encopts string
-char * hb_x264_param_unparse(const char *x264_preset,  const char *x264_tune,
-                             const char *x264_encopts, const char *h264_profile,
-                             const char *h264_level, int width, int height);
-
-#define HB_API_OLD_PRESET_GETTERS
-#ifdef  HB_API_OLD_PRESET_GETTERS
-// x264 preset/tune, qsv preset & h264 profile/level helpers
-const char * const * hb_x264_presets();
-const char * const * hb_x264_tunes();
-#ifdef USE_QSV
-const char * const * hb_qsv_presets();
-#endif
-const char * const * hb_h264_profiles();
-const char * const * hb_h264_levels();
-#endif
+char * hb_x264_param_unparse(int bit_depth, const char *x264_preset,
+                             const char *x264_tune, const char *x264_encopts,
+                             const char *h264_profile, const char *h264_level,
+                             int width, int height);
 
 // x264 option name/synonym helper
 const char * hb_x264_encopt_name( const char * name );
@@ -1293,4 +1322,12 @@ const char * hb_x264_encopt_name( const char * name );
 const char * hb_x265_encopt_name( const char * name );
 #endif
 
+#define HB_NEG_FLOAT_REG "(([-])?(([0-9]+([.,][0-9]+)?)|([.,][0-9]+))"
+#define HB_FLOAT_REG     "(([0-9]+([.,][0-9]+)?)|([.,][0-9]+))"
+#define HB_NEG_INT_REG   "(([-]?[0-9]+)"
+#define HB_INT_REG       "([0-9]+)"
+#define HB_RATIONAL_REG  "([0-9]+/[0-9]+)"
+#define HB_BOOL_REG      "(yes|no|true|false|[01])"
+#define HB_ALL_REG       "(.*)"
+
 #endif
