@@ -1,6 +1,6 @@
 /* openclwrapper.c
 
-   Copyright (c) 2003-2015 HandBrake Team
+   Copyright (c) 2003-2017 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -668,6 +668,7 @@ int hb_release_opencl_env( GPUEnv *gpu_info )
     }
 
     isInited = 0;
+    useBuffers = 0;
     gpu_info->isUserCreated = 0;
 
     return 1;
@@ -1125,6 +1126,11 @@ int hb_cl_free_mapped_buffer(cl_mem mem, unsigned char *addr)
         hb_ocl->clWaitForEvents(1, &event);
     else
         hb_log("hb_free_mapped_buffer: error %d", status);
+
+    status = hb_ocl->clReleaseMemObject(mem);
+    if (status != CL_SUCCESS)
+        hb_log("hb_free_mapped_buffer: release error %d",status);
+
     return (status == CL_SUCCESS) ? 1 : 0;
 }
 
