@@ -30,34 +30,34 @@ THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include "hb.h"
 #include "hbffmpeg.h"
-#include "libavcodec/qsv.h"
 #include "qsv_filter.h"
+#include "qsv_libav.h"
 
 struct hb_filter_private_s
 {
-    hb_job_t            *job;
-    hb_list_t           *list;
+    hb_job_t                     * job;
+    hb_buffer_list_t               list;
 
-    int                 width_in;
-    int                 height_in;
-    int                 pix_fmt;
-    int                 pix_fmt_out;
-    int                 width_out;
-    int                 height_out;
-    int                 crop[4];
-    int                 deinterlace;
-    int                 is_frc_used;
+    int                            width_in;
+    int                            height_in;
+    int                            pix_fmt;
+    int                            pix_fmt_out;
+    int                            width_out;
+    int                            height_out;
+    int                            crop[4];
+    int                            deinterlace;
+    int                            is_frc_used;
 
     // set during init, used to configure input surfaces' "area of interest"
-    mfxU16 CropX;
-    mfxU16 CropY;
-    mfxU16 CropH;
-    mfxU16 CropW;
+    mfxU16                         CropX;
+    mfxU16                         CropY;
+    mfxU16                         CropH;
+    mfxU16                         CropW;
 
-    av_qsv_space           *vpp_space;
+    hb_qsv_space                 * vpp_space;
 
     // FRC param(s)
-    mfxExtVPPFrameRateConversion    frc_config;
+    mfxExtVPPFrameRateConversion   frc_config;
 };
 
 static int hb_qsv_filter_init( hb_filter_object_t * filter,
@@ -67,24 +67,30 @@ static int hb_qsv_filter_work( hb_filter_object_t * filter,
                                hb_buffer_t ** buf_in,
                                hb_buffer_t ** buf_out );
 
-static int hb_qsv_filter_info( hb_filter_object_t * filter,
-                               hb_filter_info_t * info );
+static hb_filter_info_t * hb_qsv_filter_info( hb_filter_object_t * filter );
 
 static void hb_qsv_filter_close( hb_filter_object_t * filter );
 
+static const char qsv_filter_template[] =
+    "width=^"HB_INT_REG"$:height=^"HB_INT_REG"$:"
+    "crop-top=^"HB_INT_REG"$:crop-bottom=^"HB_INT_REG"$:"
+    "crop-left=^"HB_INT_REG"$:crop-right=^"HB_INT_REG"$:"
+    "deinterlace=^([01])$";
+
 hb_filter_object_t hb_filter_qsv =
 {
-    .id            = HB_FILTER_QSV,
-    .enforce_order = 1,
-    .name          = "Quick Sync Video VPP",
-    .settings      = NULL,
-    .init          = hb_qsv_filter_init,
-    .work          = hb_qsv_filter_work,
-    .close         = hb_qsv_filter_close,
-    .info          = hb_qsv_filter_info,
+    .id                = HB_FILTER_QSV,
+    .enforce_order     = 1,
+    .name              = "Quick Sync Video VPP",
+    .settings          = NULL,
+    .init              = hb_qsv_filter_init,
+    .work              = hb_qsv_filter_work,
+    .close             = hb_qsv_filter_close,
+    .info              = hb_qsv_filter_info,
+    .settings_template = qsv_filter_template,
 };
 
-static int filter_init( av_qsv_context* qsv, hb_filter_private_t * pv ){
+static int filter_init( hb_qsv_context* qsv, hb_filter_private_t * pv ){
     mfxStatus sts;
     int i=0;
 
@@ -92,12 +98,12 @@ static int filter_init( av_qsv_context* qsv, hb_filter_private_t * pv ){
 
 
     if(!qsv->vpp_space){
-        qsv->vpp_space = av_qsv_list_init(HAVE_THREADS);
+        qsv->vpp_space = hb_qsv_list_init(HAVE_THREADS);
     }
     if(!pv->vpp_space){
-        for(i=0; i<av_qsv_list_count(qsv->vpp_space);i++){
-            av_qsv_space *qsv_vpp = av_qsv_list_item( qsv->vpp_space, i );
-            if(qsv_vpp->type == AV_QSV_VPP_DEFAULT){
+        for(i=0; i<hb_qsv_list_count(qsv->vpp_space);i++){
+            hb_qsv_space *qsv_vpp = hb_qsv_list_item( qsv->vpp_space, i );
+            if(qsv_vpp->type == HB_QSV_VPP_DEFAULT){
                 pv->vpp_space = qsv_vpp;
                 break;
             }
@@ -105,9 +111,9 @@ static int filter_init( av_qsv_context* qsv, hb_filter_private_t * pv ){
     }
 
     if(!pv->vpp_space){
-        pv->vpp_space = calloc( 1, sizeof( av_qsv_space ));
-        pv->vpp_space->type = AV_QSV_VPP_DEFAULT;
-        av_qsv_list_add( qsv->vpp_space, pv->vpp_space );
+        pv->vpp_space = calloc( 1, sizeof( hb_qsv_space ));
+        pv->vpp_space->type = HB_QSV_VPP_DEFAULT;
+        hb_qsv_list_add( qsv->vpp_space, pv->vpp_space );
     }
     else
         if(pv->vpp_space->is_init_done ) return 1;
@@ -120,13 +126,13 @@ static int filter_init( av_qsv_context* qsv, hb_filter_private_t * pv ){
         return 2;
     }
 
-    av_qsv_add_context_usage(qsv,HAVE_THREADS);
+    hb_qsv_add_context_usage(qsv,HAVE_THREADS);
 
     // see params needed like at mediasdk-man.pdf:"Appendix A: Configuration Parameter Constraints"
     // for now - most will take from the decode
     {
-        av_qsv_space *qsv_vpp = pv->vpp_space;
-        AV_QSV_ZERO_MEMORY(qsv_vpp->m_mfxVideoParam);
+        hb_qsv_space *qsv_vpp = pv->vpp_space;
+        HB_QSV_ZERO_MEMORY(qsv_vpp->m_mfxVideoParam);
 
         if (pv->deinterlace)
         {
@@ -148,8 +154,8 @@ static int filter_init( av_qsv_context* qsv, hb_filter_private_t * pv ){
         // FrameRate is important for VPP to start with
         if( qsv->dec_space->m_mfxVideoParam.mfx.FrameInfo.FrameRateExtN == 0 &&
             qsv->dec_space->m_mfxVideoParam.mfx.FrameInfo.FrameRateExtD == 0 ){
-            qsv->dec_space->m_mfxVideoParam.mfx.FrameInfo.FrameRateExtN = pv->job->title->rate;
-            qsv->dec_space->m_mfxVideoParam.mfx.FrameInfo.FrameRateExtD = pv->job->title->rate_base;
+            qsv->dec_space->m_mfxVideoParam.mfx.FrameInfo.FrameRateExtN = pv->job->title->vrate.num;
+            qsv->dec_space->m_mfxVideoParam.mfx.FrameInfo.FrameRateExtD = pv->job->title->vrate.den;
         }
 
         /*
@@ -179,8 +185,8 @@ static int filter_init( av_qsv_context* qsv, hb_filter_private_t * pv ){
 
         qsv_vpp->m_mfxVideoParam.vpp.In.FourCC          = qsv->dec_space->m_mfxVideoParam.mfx.FrameInfo.FourCC;
         qsv_vpp->m_mfxVideoParam.vpp.In.ChromaFormat    = qsv->dec_space->m_mfxVideoParam.mfx.FrameInfo.ChromaFormat;
-        qsv_vpp->m_mfxVideoParam.vpp.In.FrameRateExtN   = pv->job->vrate;
-        qsv_vpp->m_mfxVideoParam.vpp.In.FrameRateExtD   = pv->job->vrate_base;
+        qsv_vpp->m_mfxVideoParam.vpp.In.FrameRateExtN   = pv->job->vrate.num;
+        qsv_vpp->m_mfxVideoParam.vpp.In.FrameRateExtD   = pv->job->vrate.den;
         qsv_vpp->m_mfxVideoParam.vpp.In.AspectRatioW    = qsv->dec_space->m_mfxVideoParam.mfx.FrameInfo.AspectRatioW;
         qsv_vpp->m_mfxVideoParam.vpp.In.AspectRatioH    = qsv->dec_space->m_mfxVideoParam.mfx.FrameInfo.AspectRatioH;
         qsv_vpp->m_mfxVideoParam.vpp.In.Width           = qsv->dec_space->m_mfxVideoParam.mfx.FrameInfo.Width;
@@ -192,8 +198,8 @@ static int filter_init( av_qsv_context* qsv, hb_filter_private_t * pv ){
 
         qsv_vpp->m_mfxVideoParam.vpp.Out.FourCC          = qsv->dec_space->m_mfxVideoParam.mfx.FrameInfo.FourCC;
         qsv_vpp->m_mfxVideoParam.vpp.Out.ChromaFormat    = qsv->dec_space->m_mfxVideoParam.mfx.FrameInfo.ChromaFormat;
-        qsv_vpp->m_mfxVideoParam.vpp.Out.FrameRateExtN   = pv->job->vrate;
-        qsv_vpp->m_mfxVideoParam.vpp.Out.FrameRateExtD   = pv->job->vrate_base;
+        qsv_vpp->m_mfxVideoParam.vpp.Out.FrameRateExtN   = pv->job->vrate.num;
+        qsv_vpp->m_mfxVideoParam.vpp.Out.FrameRateExtD   = pv->job->vrate.den;
         qsv_vpp->m_mfxVideoParam.vpp.Out.AspectRatioW    = qsv->dec_space->m_mfxVideoParam.mfx.FrameInfo.AspectRatioW;
         qsv_vpp->m_mfxVideoParam.vpp.Out.AspectRatioH    = qsv->dec_space->m_mfxVideoParam.mfx.FrameInfo.AspectRatioH;
         qsv_vpp->m_mfxVideoParam.vpp.Out.Width           = pv->job->qsv.enc_info.align_width;
@@ -210,32 +216,32 @@ static int filter_init( av_qsv_context* qsv, hb_filter_private_t * pv ){
         memset(&qsv_vpp->request, 0, sizeof(mfxFrameAllocRequest)*2);
 
         sts = MFXVideoVPP_QueryIOSurf(qsv->mfx_session, &qsv_vpp->m_mfxVideoParam, qsv_vpp->request );
-        AV_QSV_CHECK_RESULT(sts, MFX_ERR_NONE, sts);
+        HB_QSV_CHECK_RESULT(sts, MFX_ERR_NONE, sts);
 
         int num_surfaces_in  = qsv_vpp->request[0].NumFrameSuggested;
         int num_surfaces_out = qsv_vpp->request[1].NumFrameSuggested;
 
-        av_qsv_config *config = qsv->qsv_config;
+        hb_qsv_config *config = qsv->qsv_config;
 
 
-        qsv_vpp->surface_num = FFMIN( num_surfaces_in + num_surfaces_out + qsv_vpp->m_mfxVideoParam.AsyncDepth + config ? config->additional_buffers/2 :0 , AV_QSV_SURFACE_NUM );
+        qsv_vpp->surface_num = FFMIN( num_surfaces_in + num_surfaces_out + qsv_vpp->m_mfxVideoParam.AsyncDepth + config ? config->additional_buffers/2 :0 , HB_QSV_SURFACE_NUM );
         if(qsv_vpp->surface_num <= 0 )
-            qsv_vpp->surface_num = AV_QSV_SURFACE_NUM;
+            qsv_vpp->surface_num = HB_QSV_SURFACE_NUM;
 
         int i = 0;
         for (i = 0; i < qsv_vpp->surface_num; i++){
             qsv_vpp->p_surfaces[i] = av_mallocz( sizeof(mfxFrameSurface1) );
-            AV_QSV_CHECK_POINTER(qsv_vpp->p_surfaces[i], MFX_ERR_MEMORY_ALLOC);
+            HB_QSV_CHECK_POINTER(qsv_vpp->p_surfaces[i], MFX_ERR_MEMORY_ALLOC);
             memcpy(&(qsv_vpp->p_surfaces[i]->Info), &(qsv_vpp->m_mfxVideoParam.vpp.Out), sizeof(mfxFrameInfo));
         }
 
-        qsv_vpp->sync_num = FFMIN( qsv_vpp->surface_num, AV_QSV_SYNC_NUM );
+        qsv_vpp->sync_num = FFMIN( qsv_vpp->surface_num, HB_QSV_SYNC_NUM );
 
         for (i = 0; i < qsv_vpp->sync_num; i++){
-            qsv_vpp->p_syncp[i] = av_mallocz(sizeof(av_qsv_sync));
-            AV_QSV_CHECK_POINTER(qsv_vpp->p_syncp[i], MFX_ERR_MEMORY_ALLOC);
+            qsv_vpp->p_syncp[i] = av_mallocz(sizeof(hb_qsv_sync));
+            HB_QSV_CHECK_POINTER(qsv_vpp->p_syncp[i], MFX_ERR_MEMORY_ALLOC);
             qsv_vpp->p_syncp[i]->p_sync = av_mallocz(sizeof(mfxSyncPoint));
-            AV_QSV_CHECK_POINTER(qsv_vpp->p_syncp[i]->p_sync, MFX_ERR_MEMORY_ALLOC);
+            HB_QSV_CHECK_POINTER(qsv_vpp->p_syncp[i]->p_sync, MFX_ERR_MEMORY_ALLOC);
         }
 /*
     about available VPP filters, see "Table 4 Configurable VPP filters", mediasdk-man.pdf
@@ -297,7 +303,7 @@ static int filter_init( av_qsv_context* qsv, hb_filter_private_t * pv ){
         qsv_vpp->m_mfxVideoParam.NumExtParam        = qsv_vpp->p_ext_param_num = 1 + pv->is_frc_used;
 
         qsv_vpp->p_ext_params = av_mallocz(sizeof(mfxExtBuffer *)*qsv_vpp->p_ext_param_num);
-        AV_QSV_CHECK_POINTER(qsv_vpp->p_ext_params, MFX_ERR_MEMORY_ALLOC);
+        HB_QSV_CHECK_POINTER(qsv_vpp->p_ext_params, MFX_ERR_MEMORY_ALLOC);
 
         qsv_vpp->m_mfxVideoParam.ExtParam           = qsv_vpp->p_ext_params;
 
@@ -324,8 +330,8 @@ static int filter_init( av_qsv_context* qsv, hb_filter_private_t * pv ){
 
         sts = MFXVideoVPP_Init(qsv->mfx_session, &qsv_vpp->m_mfxVideoParam);
 
-        AV_QSV_IGNORE_MFX_STS(sts, MFX_WRN_PARTIAL_ACCELERATION);
-        AV_QSV_CHECK_RESULT(sts, MFX_ERR_NONE, sts);
+        HB_QSV_IGNORE_MFX_STS(sts, MFX_WRN_PARTIAL_ACCELERATION);
+        HB_QSV_CHECK_RESULT(sts, MFX_ERR_NONE, sts);
 
         qsv_vpp->is_init_done = 1;
     }
@@ -339,51 +345,51 @@ static int hb_qsv_filter_init( hb_filter_object_t * filter,
     filter->private_data = calloc( 1, sizeof(struct hb_filter_private_s) );
     hb_filter_private_t * pv = filter->private_data;
 
-    pv->list = hb_list_init();
+    hb_buffer_list_clear(&pv->list);
     // list of init params provided at work.c:~700
-    pv->width_in  = init->width;
-    pv->height_in = init->height;
-    pv->width_out = init->width;
-    pv->height_out = init->height;
+    pv->width_in  = init->geometry.width;
+    pv->height_in = init->geometry.height;
+    pv->width_out = init->geometry.width;
+    pv->height_out = init->geometry.height;
     memcpy( pv->crop, init->crop, sizeof( int[4] ) );
 
-    if (filter->settings != NULL)
-    {
-        sscanf(filter->settings, "%d:%d:%d:%d:%d:%d_dei:%d",
-               &pv->width_out, &pv->height_out,
-               &pv->crop[0], &pv->crop[1], &pv->crop[2], &pv->crop[3],
-               &pv->deinterlace);
-    }
+    hb_dict_extract_int(&pv->width_out, filter->settings, "width");
+    hb_dict_extract_int(&pv->height_out, filter->settings, "height");
+    hb_dict_extract_int(&pv->crop[0], filter->settings, "crop-top");
+    hb_dict_extract_int(&pv->crop[1], filter->settings, "crop-bottom");
+    hb_dict_extract_int(&pv->crop[2], filter->settings, "crop-left");
+    hb_dict_extract_int(&pv->crop[3], filter->settings, "crop-right");
+    hb_dict_extract_bool(&pv->deinterlace, filter->settings, "deinterlace");
 
     pv->job = init->job;
 
     // will be later as more params will be known
     // filter_init(pv->job->qsv, pv);
 
-    // just passing
-    init->vrate = init->vrate;
-    init->vrate_base = init->vrate_base;
-
     // framerate shaping not yet supported
     init->cfr = 0;
 
     init->pix_fmt = pv->pix_fmt;
-    init->width = pv->width_out;
-    init->height = pv->height_out;
+    init->geometry.width = pv->width_out;
+    init->geometry.height = pv->height_out;
     memcpy( init->crop, pv->crop, sizeof( int[4] ) );
 
     return 0;
 }
 
-static int hb_qsv_filter_info( hb_filter_object_t * filter,
-                               hb_filter_info_t * info )
+static hb_filter_info_t * hb_qsv_filter_info( hb_filter_object_t * filter )
 {
-
     hb_filter_private_t *pv = filter->private_data;
-    if (pv == NULL)
-        return -1;
+    hb_filter_info_t    * info;
 
-    sprintf(info->human_readable_desc,
+    if( !pv )
+        return NULL;
+
+    info = calloc(1, sizeof(hb_filter_info_t));
+    info->human_readable_desc = malloc(128);
+    info->human_readable_desc[0] = 0;
+
+    snprintf(info->human_readable_desc, 128,
             "source: %d * %d, crop (%d/%d/%d/%d): %d * %d, scale: %d * %d",
             pv->width_in, pv->height_in,
             pv->crop[0], pv->crop[1], pv->crop[2], pv->crop[3],
@@ -393,24 +399,24 @@ static int hb_qsv_filter_info( hb_filter_object_t * filter,
 
     if (pv->deinterlace)
     {
-        sprintf(info->human_readable_desc + strlen(info->human_readable_desc),
-                ", deinterlace");
+        int len = strlen(info->human_readable_desc);
+        snprintf(info->human_readable_desc + len, 128 - len, ", deinterlace");
     }
 
-    return 0;
+    return info;
 }
 
-void qsv_filter_close( av_qsv_context* qsv, AV_QSV_STAGE_TYPE vpp_type ){
+void qsv_filter_close( hb_qsv_context* qsv, HB_QSV_STAGE_TYPE vpp_type ){
     int i = 0;
-    av_qsv_space* vpp_space = 0;
+    hb_qsv_space* vpp_space = 0;
 
     if(qsv && qsv->is_context_active && qsv->vpp_space)
-    for(i=av_qsv_list_count( qsv->vpp_space);i>0;i--){
+    for(i=hb_qsv_list_count( qsv->vpp_space);i>0;i--){
 
-        vpp_space = av_qsv_list_item( qsv->vpp_space, i-1 );
+        vpp_space = hb_qsv_list_item( qsv->vpp_space, i-1 );
         if( vpp_space->type == vpp_type && vpp_space->is_init_done){
 
-            hb_log( "qsv_filter[%s] done: max_surfaces: %u/%u , max_syncs: %u/%u", ((vpp_type == AV_QSV_VPP_DEFAULT)?"Default": "User") ,vpp_space->surface_num_max_used, vpp_space->surface_num, vpp_space->sync_num_max_used, vpp_space->sync_num );
+            hb_log( "qsv_filter[%s] done: max_surfaces: %u/%u , max_syncs: %u/%u", ((vpp_type == HB_QSV_VPP_DEFAULT)?"Default": "User") ,vpp_space->surface_num_max_used, vpp_space->surface_num, vpp_space->sync_num_max_used, vpp_space->sync_num );
 
             for (i = 0; i < vpp_space->surface_num; i++){
                 av_freep(&vpp_space->p_surfaces[i]);
@@ -427,9 +433,9 @@ void qsv_filter_close( av_qsv_context* qsv, AV_QSV_STAGE_TYPE vpp_type ){
             }
             vpp_space->sync_num = 0;
 
-            av_qsv_list_rem(qsv->vpp_space,vpp_space);
-            if( av_qsv_list_count(qsv->vpp_space) == 0 )
-                av_qsv_list_close(&qsv->vpp_space);
+            hb_qsv_list_rem(qsv->vpp_space,vpp_space);
+            if( hb_qsv_list_count(qsv->vpp_space) == 0 )
+                hb_qsv_list_close(&qsv->vpp_space);
 
             vpp_space->is_init_done = 0;
             break;
@@ -439,7 +445,6 @@ void qsv_filter_close( av_qsv_context* qsv, AV_QSV_STAGE_TYPE vpp_type ){
 
 static void hb_qsv_filter_close( hb_filter_object_t * filter )
 {
-    int i = 0;
     hb_filter_private_t * pv = filter->private_data;
 
     if ( !pv )
@@ -447,37 +452,37 @@ static void hb_qsv_filter_close( hb_filter_object_t * filter )
         return;
     }
 
-    av_qsv_context* qsv = pv->job->qsv.ctx;
-    if(qsv && qsv->vpp_space && av_qsv_list_count(qsv->vpp_space) > 0){
+    hb_qsv_context* qsv = pv->job->qsv.ctx;
+    if(qsv && qsv->vpp_space && hb_qsv_list_count(qsv->vpp_space) > 0){
 
         // closing local stuff
-        qsv_filter_close(qsv,AV_QSV_VPP_DEFAULT);
+        qsv_filter_close(qsv,HB_QSV_VPP_DEFAULT);
 
         // closing the commong stuff
-        av_qsv_context_clean(qsv);
+        hb_qsv_context_clean(qsv);
     }
-    hb_list_close(&pv->list);
+    hb_buffer_list_close(&pv->list);
     free( pv );
     filter->private_data = NULL;
 }
 
-int process_frame(av_qsv_list* received_item, av_qsv_context* qsv, hb_filter_private_t * pv ){
+int process_frame(hb_qsv_list* received_item, hb_qsv_context* qsv, hb_filter_private_t * pv ){
 
     // 1 if have results , 0 - otherwise
     int ret = 1;
 
     mfxStatus sts = MFX_ERR_NONE;
     mfxFrameSurface1 *work_surface = NULL;
-    av_qsv_stage* stage = 0;
+    hb_qsv_stage* stage = 0;
 
-    av_qsv_space *qsv_vpp = pv->vpp_space;
+    hb_qsv_space *qsv_vpp = pv->vpp_space;
 
     if(received_item){
-        stage = av_qsv_get_last_stage( received_item );
+        stage = hb_qsv_get_last_stage( received_item );
         work_surface = stage->out.p_surface;
     }
 
-    int sync_idx = av_qsv_get_free_sync(qsv_vpp, qsv);
+    int sync_idx = hb_qsv_get_free_sync(qsv_vpp, qsv);
     int surface_idx = -1;
 
     for(;;)
@@ -489,7 +494,7 @@ int process_frame(av_qsv_list* received_item, av_qsv_context* qsv, hb_filter_pri
                 break;
             }
             if( sts == MFX_ERR_MORE_SURFACE || sts == MFX_ERR_NONE )
-               surface_idx = av_qsv_get_free_surface(qsv_vpp, qsv,  &(qsv_vpp->m_mfxVideoParam.vpp.Out), QSV_PART_ANY);
+               surface_idx = hb_qsv_get_free_surface(qsv_vpp, qsv,  &(qsv_vpp->m_mfxVideoParam.vpp.Out), QSV_PART_ANY);
             if (surface_idx == -1) {
                 hb_error("qsv: Not enough resources allocated for QSV filter");
                 ret = 0;
@@ -507,11 +512,11 @@ int process_frame(av_qsv_list* received_item, av_qsv_context* qsv, hb_filter_pri
 
             if( MFX_ERR_MORE_DATA == sts ){
                 if(!qsv_vpp->pending){
-                    qsv_vpp->pending = av_qsv_list_init(0);
+                    qsv_vpp->pending = hb_qsv_list_init(0);
                 }
 
                 // if we have no results, we should not miss resource(s)
-                av_qsv_list_add( qsv_vpp->pending, received_item);
+                hb_qsv_list_add( qsv_vpp->pending, received_item);
 
                 ff_qsv_atomic_dec(&qsv_vpp->p_syncp[sync_idx]->in_use);
 
@@ -533,37 +538,37 @@ int process_frame(av_qsv_list* received_item, av_qsv_context* qsv, hb_filter_pri
                    ff_qsv_atomic_inc(&qsv_vpp->p_surfaces[surface_idx]->Data.Locked);
             }
 
-            AV_QSV_CHECK_RESULT(sts, MFX_ERR_NONE, sts);
+            HB_QSV_CHECK_RESULT(sts, MFX_ERR_NONE, sts);
 
             if (MFX_ERR_NONE <= sts ) // repeat the call if warning and no output
             {
                 if (MFX_WRN_DEVICE_BUSY == sts){
-                    av_qsv_sleep(10); // wait if device is busy
+                    hb_qsv_sleep(10); // wait if device is busy
                     continue;
                 }
 
                 // shouldnt be a case but drain
                 if(stage){
-                        av_qsv_stage* new_stage = av_qsv_stage_init();
+                        hb_qsv_stage* new_stage = hb_qsv_stage_init();
 
-                        new_stage->type = AV_QSV_VPP_DEFAULT;
+                        new_stage->type = HB_QSV_VPP_DEFAULT;
                         new_stage->in.p_surface  =  work_surface;
                         new_stage->out.p_surface = qsv_vpp->p_surfaces[surface_idx];
                         new_stage->out.sync      = qsv_vpp->p_syncp[sync_idx];
-                        av_qsv_add_stagee( &received_item, new_stage,HAVE_THREADS );
+                        hb_qsv_add_stagee( &received_item, new_stage,HAVE_THREADS );
 
                         // add pending resources for the proper reclaim later
                         if( qsv_vpp->pending ){
-                            if( av_qsv_list_count(qsv_vpp->pending)>0 ){
+                            if( hb_qsv_list_count(qsv_vpp->pending)>0 ){
                                 new_stage->pending = qsv_vpp->pending;
                 }
                             qsv_vpp->pending = 0;
 
                             // making free via decrement for all pending
                             int i = 0;
-                            for (i = av_qsv_list_count(new_stage->pending); i > 0; i--){
-                                av_qsv_list *atom_list = av_qsv_list_item(new_stage->pending, i-1);
-                                av_qsv_stage *stage = av_qsv_get_last_stage( atom_list );
+                            for (i = hb_qsv_list_count(new_stage->pending); i > 0; i--){
+                                hb_qsv_list *atom_list = hb_qsv_list_item(new_stage->pending, i-1);
+                                hb_qsv_stage *stage = hb_qsv_get_last_stage( atom_list );
                                 mfxFrameSurface1 *work_surface = stage->out.p_surface;
                                 if (work_surface)
                                    ff_qsv_atomic_dec(&work_surface->Data.Locked);
@@ -594,90 +599,76 @@ static int hb_qsv_filter_work( hb_filter_object_t * filter,
     hb_buffer_t * out = *buf_out;
     int sts = 0;
 
-    av_qsv_context* qsv = pv->job->qsv.ctx;
-
     if ( !pv )
     {
         *buf_out = in;
         *buf_in = NULL;
         return HB_FILTER_OK;
     }
+    
+    hb_qsv_context* qsv = pv->job->qsv.ctx;
 
-    while(1){
+    while(1)
+    {
         int ret = filter_init(qsv,pv);
         if(ret >= 2)
-            av_qsv_sleep(1);
+            hb_qsv_sleep(1);
         else
             break;
     }
 
     *buf_in = NULL;
 
-    if ( in->size <= 0 )
+    if (in->s.flags & HB_BUF_FLAG_EOF)
     {
-        while(1){
+        while(1)
+        {
             sts = process_frame(in->qsv_details.qsv_atom, qsv, pv);
             if(sts)
-                hb_list_add(pv->list,in);
+                hb_buffer_list_append(&pv->list, in);
             else
                 break;
         }
 
-        hb_list_add( pv->list, in );
-        *buf_out = link_buf_list( pv );
+        hb_buffer_list_append(&pv->list, in);
+        *buf_out = hb_buffer_list_clear(&pv->list);
         return HB_FILTER_DONE;
     }
 
     sts = process_frame(in->qsv_details.qsv_atom, qsv, pv);
 
-    if(sts){
-        hb_list_add(pv->list,in);
+    if(sts)
+    {
+        hb_buffer_list_append(&pv->list, in);
     }
 
-    if( hb_list_count(pv->list) ){
-        *buf_out = hb_list_item(pv->list,0);
-        out = *buf_out;
-        if(pv->is_frc_used && out)
+    out = *buf_out = hb_buffer_list_rem_head(&pv->list);
+    if (pv->is_frc_used && out != NULL)
+    {
+        if (out->qsv_details.qsv_atom)
         {
-            mfxStatus sts = MFX_ERR_NONE;
-                if(out->qsv_details.qsv_atom){
-                    av_qsv_stage* stage = av_qsv_get_last_stage( out->qsv_details.qsv_atom );
-                    mfxFrameSurface1 *work_surface = stage->out.p_surface;
+            hb_qsv_stage* stage;
+            mfxFrameSurface1 *work_surface;
+            int64_t duration;
+            hb_qsv_space *qsv_vpp;
 
-                    av_qsv_wait_on_sync( qsv,stage );
+            stage        = hb_qsv_get_last_stage(out->qsv_details.qsv_atom);
+            work_surface = stage->out.p_surface;
 
-                    av_qsv_space *qsv_vpp = pv->vpp_space;
-                    int64_t duration  = ((double)qsv_vpp->m_mfxVideoParam.vpp.Out.FrameRateExtD/(double)qsv_vpp->m_mfxVideoParam.vpp.Out.FrameRateExtN ) * 90000.;
-                    out->s.start = work_surface->Data.TimeStamp;
-                    out->s.stop = work_surface->Data.TimeStamp + duration;
-                }
+            hb_qsv_wait_on_sync( qsv,stage );
+
+            qsv_vpp  = pv->vpp_space;
+            duration =
+                ((double)qsv_vpp->m_mfxVideoParam.vpp.Out.FrameRateExtD /
+                 (double)qsv_vpp->m_mfxVideoParam.vpp.Out.FrameRateExtN ) *
+                90000.;
+            out->s.start = work_surface->Data.TimeStamp;
+            out->s.stop  = work_surface->Data.TimeStamp + duration;
         }
-        hb_list_rem(pv->list,*buf_out);
     }
-    else
-        *buf_out = NULL;
 
     return HB_FILTER_OK;
 }
 
-// see devavcode.c
-hb_buffer_t *link_buf_list( hb_filter_private_t *pv )
-{
-        hb_buffer_t *head = hb_list_item( pv->list, 0 );
-
-        if ( head )
-        {
-            hb_list_rem( pv->list, head );
-            hb_buffer_t *last = head, *buf;
-            while ( ( buf = hb_list_item( pv->list, 0 ) ) != NULL )
-            {
-                hb_list_rem( pv->list, buf );
-                last->next = buf;
-                last = buf;
-            }
-        }
-    return head;
-}
-
 #endif // USE_QSV
 
