@@ -1,6 +1,6 @@
 /* rendersub.c
 
-   Copyright (c) 2003-2015 HandBrake Team
+   Copyright (c) 2003-2017 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -11,28 +11,33 @@
 #include "hbffmpeg.h"
 #include <ass/ass.h>
 
+#define ABS(a) ((a) > 0 ? (a) : (-(a)))
+
 struct hb_filter_private_s
 {
     // Common
-    int               crop[4];
-    int               type;
+    int                 crop[4];
+    int                 type;
+    struct SwsContext * sws;
+    int                 sws_width;
+    int                 sws_height;
 
     // VOBSUB
-    hb_list_t       * sub_list; // List of active subs
+    hb_list_t         * sub_list; // List of active subs
 
     // SSA
-    ASS_Library     * ssa;
-    ASS_Renderer    * renderer;
-    ASS_Track       * ssaTrack;
-    uint8_t           script_initialized;
+    ASS_Library       * ssa;
+    ASS_Renderer      * renderer;
+    ASS_Track         * ssaTrack;
+    uint8_t             script_initialized;
 
     // SRT
-    int               line;
-    hb_buffer_t     * current_sub;
+    int                 line;
+    hb_buffer_t       * current_sub;
 };
 
 // VOBSUB
-static int vobsub_init( hb_filter_object_t * filter, hb_filter_init_t * init );
+static int vobsub_post_init( hb_filter_object_t * filter, hb_job_t * job );
 
 static int vobsub_work( hb_filter_object_t * filter,
                         hb_buffer_t ** buf_in,
@@ -42,7 +47,7 @@ static void vobsub_close( hb_filter_object_t * filter );
 
 
 // SSA
-static int ssa_init( hb_filter_object_t * filter, hb_filter_init_t * init );
+static int ssa_post_init( hb_filter_object_t * filter, hb_job_t * job );
 
 static int ssa_work( hb_filter_object_t * filter,
                      hb_buffer_t ** buf_in,
@@ -52,7 +57,8 @@ static void ssa_close( hb_filter_object_t * filter );
 
 
 // SRT
-static int textsub_init( hb_filter_object_t * filter, hb_filter_init_t * init );
+static int textsub_post_init( hb_filter_object_t * filter, hb_job_t * job );
+static int cc608sub_post_init( hb_filter_object_t * filter, hb_job_t * job );
 
 static int textsub_work( hb_filter_object_t * filter,
                      hb_buffer_t ** buf_in,
@@ -62,7 +68,7 @@ static void textsub_close( hb_filter_object_t * filter );
 
 
 // PGS
-static int pgssub_init ( hb_filter_object_t * filter, hb_filter_init_t * init );
+static int pgssub_post_init( hb_filter_object_t * filter, hb_job_t * job );
 
 static int pgssub_work ( hb_filter_object_t * filter,
                       hb_buffer_t ** buf_in,
@@ -75,6 +81,8 @@ static void pgssub_close( hb_filter_object_t * filter );
 static int hb_rendersub_init( hb_filter_object_t * filter,
                                  hb_filter_init_t * init );
 
+static int hb_rendersub_post_init( hb_filter_object_t * filter, hb_job_t *job );
+
 static int hb_rendersub_work( hb_filter_object_t * filter,
                                  hb_buffer_t ** buf_in,
                                  hb_buffer_t ** buf_out );
@@ -88,6 +96,7 @@ hb_filter_object_t hb_filter_render_sub =
     .name          = "Subtitle renderer",
     .settings      = NULL,
     .init          = hb_rendersub_init,
+    .post_init     = hb_rendersub_post_init,
     .work          = hb_rendersub_work,
     .close         = hb_rendersub_close,
 };
@@ -178,84 +187,144 @@ static void blend( hb_buffer_t *dst, hb_buffer_t *src, int left, int top )
 // as the original title diminsions
 static void ApplySub( hb_filter_private_t * pv, hb_buffer_t * buf, hb_buffer_t * sub )
 {
-    int top, left, margin_top, margin_percent;
+    blend( buf, sub, sub->f.x, sub->f.y );
+}
 
-    if ( !pv->ssa )
+static hb_buffer_t * ScaleSubtitle(hb_filter_private_t *pv,
+                                   hb_buffer_t *sub, hb_buffer_t *buf)
+{
+    hb_buffer_t * scaled;
+    double xfactor = 1., yfactor = 1.;
+
+    // Do we need to rescale subtitles?
+    if (sub->f.window_width > 0 && sub->f.window_height > 0)
     {
-        /*
-         * Percent of height of picture that form a margin that subtitles
-         * should not be displayed within.
-         */
-        margin_percent = 2;
 
-        /*
-         * If necessary, move the subtitle so it is not in a cropped zone.
-         * When it won't fit, we center it so we lose as much on both ends.
-         * Otherwise we try to leave a 20px or 2% margin around it.
-         */
-        margin_top = ( ( buf->f.height - pv->crop[0] - pv->crop[1] ) *
-                       margin_percent ) / 100;
-
-        if( margin_top > 20 )
+        // TODO: Factor aspect ratio
+        // For now, assume subtitle and video PAR is the same.
+        xfactor     = (double)buf->f.width  / sub->f.window_width;
+        yfactor     = (double)buf->f.height / sub->f.window_height;
+        // The video may have been cropped.  This will make xfactor != yfactor
+        // even though video and subtitles are the same PAR.  So use the
+        // larger of as the scale factor.
+        if (xfactor > yfactor)
         {
-            /*
-             * A maximum margin of 20px regardless of height of the picture.
-             */
-            margin_top = 20;
-        }
-
-        if( sub->f.height > buf->f.height - pv->crop[0] - pv->crop[1] -
-            ( margin_top * 2 ) )
-        {
-            /*
-             * The subtitle won't fit in the cropped zone, so center
-             * it vertically so we fit in as much as we can.
-             */
-            top = pv->crop[0] + ( buf->f.height - pv->crop[0] -
-                                          pv->crop[1] - sub->f.height ) / 2;
-        }
-        else if( sub->f.y < pv->crop[0] + margin_top )
-        {
-            /*
-             * The subtitle fits in the cropped zone, but is currently positioned
-             * within our top margin, so move it outside of our margin.
-             */
-            top = pv->crop[0] + margin_top;
-        }
-        else if( sub->f.y > buf->f.height - pv->crop[1] - margin_top - sub->f.height )
-        {
-            /*
-             * The subtitle fits in the cropped zone, and is not within the top
-             * margin but is within the bottom margin, so move it to be above
-             * the margin.
-             */
-            top = buf->f.height - pv->crop[1] - margin_top - sub->f.height;
+            yfactor = xfactor;
         }
         else
         {
-            /*
-             * The subtitle is fine where it is.
-             */
-            top = sub->f.y;
+            xfactor = yfactor;
         }
+    }
+    if (ABS(xfactor - 1) > 0.01 || ABS(yfactor - 1) > 0.01)
+    {
+        uint8_t * in_data[4], * out_data[4];
+        int       in_stride[4], out_stride[4];
+        int       width, height;
 
-        if( sub->f.width > buf->f.width - pv->crop[2] - pv->crop[3] - 40 )
-            left = pv->crop[2] + ( buf->f.width - pv->crop[2] -
-                    pv->crop[3] - sub->f.width ) / 2;
-        else if( sub->f.x < pv->crop[2] + 20 )
-            left = pv->crop[2] + 20;
-        else if( sub->f.x > buf->f.width - pv->crop[3] - 20 - sub->f.width )
-            left = buf->f.width - pv->crop[3] - 20 - sub->f.width;
-        else
-            left = sub->f.x;
+        width       = sub->f.width  * xfactor;
+        height      = sub->f.height * yfactor;
+        scaled      = hb_frame_buffer_init(AV_PIX_FMT_YUVA420P, width, height);
+        scaled->f.x = sub->f.x * xfactor;
+        scaled->f.y = sub->f.y * yfactor;
+
+        hb_picture_fill(in_data,  in_stride,  sub);
+        hb_picture_fill(out_data, out_stride, scaled);
+
+        if (pv->sws        == NULL   ||
+            pv->sws_width  != width  ||
+            pv->sws_height != height)
+        {
+            if (pv->sws!= NULL)
+                sws_freeContext(pv->sws);
+            pv->sws = hb_sws_get_context(
+                                sub->f.width, sub->f.height, sub->f.fmt,
+                                scaled->f.width, scaled->f.height, sub->f.fmt,
+                                SWS_LANCZOS|SWS_ACCURATE_RND, SWS_CS_DEFAULT);
+            pv->sws_width   = width;
+            pv->sws_height  = height;
+        }
+        sws_scale(pv->sws, (const uint8_t* const *)in_data, in_stride,
+                  0, sub->f.height, out_data, out_stride);
     }
     else
     {
-        top = sub->f.y;
-        left = sub->f.x;
+        scaled = hb_buffer_dup(sub);
     }
 
-    blend( buf, sub, left, top );
+    int top, left, margin_top, margin_percent;
+
+    /*
+     * Percent of height of picture that form a margin that subtitles
+     * should not be displayed within.
+     */
+    margin_percent = 2;
+
+    /*
+     * If necessary, move the subtitle so it is not in a cropped zone.
+     * When it won't fit, we center it so we lose as much on both ends.
+     * Otherwise we try to leave a 20px or 2% margin around it.
+     */
+    margin_top = ( ( buf->f.height - pv->crop[0] - pv->crop[1] ) *
+                   margin_percent ) / 100;
+
+    if( margin_top > 20 )
+    {
+        /*
+         * A maximum margin of 20px regardless of height of the picture.
+         */
+        margin_top = 20;
+    }
+
+    if( scaled->f.height > buf->f.height - pv->crop[0] - pv->crop[1] -
+        ( margin_top * 2 ) )
+    {
+        /*
+         * The subtitle won't fit in the cropped zone, so center
+         * it vertically so we fit in as much as we can.
+         */
+        top = pv->crop[0] + ( buf->f.height - pv->crop[0] -
+                                      pv->crop[1] - scaled->f.height ) / 2;
+    }
+    else if( scaled->f.y < pv->crop[0] + margin_top )
+    {
+        /*
+         * The subtitle fits in the cropped zone, but is currently positioned
+         * within our top margin, so move it outside of our margin.
+         */
+        top = pv->crop[0] + margin_top;
+    }
+    else if( scaled->f.y > buf->f.height - pv->crop[1] - margin_top - scaled->f.height )
+    {
+        /*
+         * The subtitle fits in the cropped zone, and is not within the top
+         * margin but is within the bottom margin, so move it to be above
+         * the margin.
+         */
+        top = buf->f.height - pv->crop[1] - margin_top - scaled->f.height;
+    }
+    else
+    {
+        /*
+         * The subtitle is fine where it is.
+         */
+        top = scaled->f.y;
+    }
+
+    if( scaled->f.width > buf->f.width - pv->crop[2] - pv->crop[3] - 40 )
+        left = pv->crop[2] + ( buf->f.width - pv->crop[2] -
+                pv->crop[3] - scaled->f.width ) / 2;
+    else if( scaled->f.x < pv->crop[2] + 20 )
+        left = pv->crop[2] + 20;
+    else if( scaled->f.x > buf->f.width - pv->crop[3] - 20 - scaled->f.width )
+        left = buf->f.width - pv->crop[3] - 20 - scaled->f.width;
+    else
+        left = scaled->f.x;
+
+    scaled->f.x = left;
+    scaled->f.y = top;
+
+    return scaled;
 }
 
 // Assumes that the input buffer has the same dimensions
@@ -286,7 +355,9 @@ static void ApplyVOBSubs( hb_filter_private_t * pv, hb_buffer_t * buf )
             // after it.  Render the subtitle into the frame.
             while ( sub )
             {
-                ApplySub( pv, buf, sub );
+                hb_buffer_t *scaled = ScaleSubtitle(pv, sub, buf);
+                ApplySub( pv, buf, scaled );
+                hb_buffer_close(&scaled);
                 sub = sub->next;
             }
             ii++;
@@ -299,8 +370,7 @@ static void ApplyVOBSubs( hb_filter_private_t * pv, hb_buffer_t * buf )
     }
 }
 
-static int vobsub_init( hb_filter_object_t * filter,
-                        hb_filter_init_t * init )
+static int vobsub_post_init( hb_filter_object_t * filter, hb_job_t * job )
 {
     hb_filter_private_t * pv = filter->private_data;
 
@@ -333,7 +403,7 @@ static int vobsub_work( hb_filter_object_t * filter,
     hb_buffer_t * in = *buf_in;
     hb_buffer_t * sub;
 
-    if ( in->size <= 0 )
+    if (in->s.flags & HB_BUF_FLAG_EOF)
     {
         *buf_in = NULL;
         *buf_out = in;
@@ -344,6 +414,11 @@ static int vobsub_work( hb_filter_object_t * filter,
     // subtitle list
     while( ( sub = hb_fifo_get( filter->subtitle->fifo_out ) ) )
     {
+        if (sub->s.flags & HB_BUF_FLAG_EOF)
+        {
+            hb_buffer_close(&sub);
+            break;
+        }
         hb_list_add( pv->sub_list, sub );
     }
 
@@ -448,8 +523,7 @@ static void ssa_log(int level, const char *fmt, va_list args, void *data)
     }
 }
 
-static int ssa_init( hb_filter_object_t * filter,
-                     hb_filter_init_t * init )
+static int ssa_post_init( hb_filter_object_t * filter, hb_job_t * job )
 {
     hb_filter_private_t * pv = filter->private_data;
 
@@ -463,13 +537,14 @@ static int ssa_init( hb_filter_object_t * filter,
     ass_set_message_cb( pv->ssa, ssa_log, NULL );
 
     // Load embedded fonts
-    hb_list_t * list_attachment = init->job->list_attachment;
+    hb_list_t * list_attachment = job->list_attachment;
     int i;
     for ( i = 0; i < hb_list_count(list_attachment); i++ )
     {
         hb_attachment_t * attachment = hb_list_item( list_attachment, i );
 
-        if ( attachment->type == FONT_TTF_ATTACH )
+        if ( attachment->type == FONT_TTF_ATTACH ||
+             attachment->type == FONT_OTF_ATTACH )
         {
             ass_add_font(
                 pv->ssa,
@@ -509,11 +584,11 @@ static int ssa_init( hb_filter_object_t * filter,
         return 1;
     }
 
-    int width = init->width - ( pv->crop[2] + pv->crop[3] );
-    int height = init->height - ( pv->crop[0] + pv->crop[1] );
+    int height = job->title->geometry.height - job->crop[0] - job->crop[1];
+    int width = job->title->geometry.width - job->crop[2] - job->crop[3];
     ass_set_frame_size( pv->renderer, width, height);
 
-    double par = (double)init->par_width / init->par_height;
+    double par = (double)job->par.num / job->par.den;
     ass_set_aspect_ratio( pv->renderer, 1, par );
 
     return 0;
@@ -550,7 +625,7 @@ static int ssa_work( hb_filter_object_t * filter,
     if (!pv->script_initialized)
     {
         // NOTE: The codec extradata is expected to be in MKV format
-        // I would like to initialize this in ssa_init, but when we are
+        // I would like to initialize this in ssa_post_init, but when we are
         // transcoding text subtitles to SSA, the extradata does not
         // get initialized until the decoder is initialized.  Since
         // decoder initialization happens after filter initialization,
@@ -560,7 +635,7 @@ static int ssa_work( hb_filter_object_t * filter,
                                   filter->subtitle->extradata_size);
         pv->script_initialized = 1;
     }
-    if ( in->size <= 0 )
+    if (in->s.flags & HB_BUF_FLAG_EOF)
     {
         *buf_in = NULL;
         *buf_out = in;
@@ -571,6 +646,11 @@ static int ssa_work( hb_filter_object_t * filter,
     // subtitle list
     while( ( sub = hb_fifo_get( filter->subtitle->fifo_out ) ) )
     {
+        if (sub->s.flags & HB_BUF_FLAG_EOF)
+        {
+            hb_buffer_close(&sub);
+            break;
+        }
         // Parse MKV-SSA packet
         // SSA subtitles always have an explicit stop time, so we
         // do not need to do special processing for stop == AV_NOPTS_VALUE
@@ -587,18 +667,29 @@ static int ssa_work( hb_filter_object_t * filter,
     return HB_FILTER_OK;
 }
 
-static int textsub_init( hb_filter_object_t * filter,
-                     hb_filter_init_t * init )
+static int cc608sub_post_init( hb_filter_object_t * filter, hb_job_t * job )
 {
-    hb_filter_private_t * pv = filter->private_data;
-
-    int width = init->width - pv->crop[2] - pv->crop[3];
-    int height = init->height - pv->crop[0] - pv->crop[1];
-
     // Text subtitles for which we create a dummy ASS header need
     // to have the header rewritten with the correct dimensions.
-    hb_subtitle_add_ssa_header(filter->subtitle, width, height);
-    return ssa_init(filter, init);
+    int height = job->title->geometry.height - job->crop[0] - job->crop[1];
+    int width = job->title->geometry.width - job->crop[2] - job->crop[3];
+    int safe_height = 0.8 * job->title->geometry.height;
+    // Use fixed widht font for CC
+    hb_subtitle_add_ssa_header(filter->subtitle, "Courier New",
+                               .08 * safe_height, width, height);
+    return ssa_post_init(filter, job);
+}
+
+static int textsub_post_init( hb_filter_object_t * filter, hb_job_t * job )
+{
+    // Text subtitles for which we create a dummy ASS header need
+    // to have the header rewritten with the correct dimensions.
+    int height = job->title->geometry.height - job->crop[0] - job->crop[1];
+    int width = job->title->geometry.width - job->crop[2] - job->crop[3];
+    hb_subtitle_add_ssa_header(filter->subtitle, "Arial",
+                               .066 * job->title->geometry.height,
+                               width, height);
+    return ssa_post_init(filter, job);
 }
 
 static void textsub_close( hb_filter_object_t * filter )
@@ -645,7 +736,7 @@ static int textsub_work(hb_filter_object_t * filter,
         pv->script_initialized = 1;
     }
 
-    if (in->size <= 0)
+    if (in->s.flags & HB_BUF_FLAG_EOF)
     {
         *buf_in = NULL;
         *buf_out = in;
@@ -658,6 +749,20 @@ static int textsub_work(hb_filter_object_t * filter,
     // subtitle list
     while ((sub = hb_fifo_get(filter->subtitle->fifo_out)))
     {
+        if (sub->s.flags & HB_BUF_FLAG_EOF)
+        {
+            hb_buffer_close(&sub);
+            if (pv->current_sub != NULL)
+            {
+                // Make us some duration for final sub
+                pv->current_sub->s.stop = pv->current_sub->s.start +
+                                          90000LL * 10;
+                process_sub(pv, pv->current_sub);
+                hb_buffer_close(&pv->current_sub);
+            }
+            break;
+        }
+
         // libass expects times in ms.  So to make the math easy,
         // convert to ms immediately.
         sub->s.start /= 90;
@@ -680,9 +785,9 @@ static int textsub_work(hb_filter_object_t * filter,
             process_sub(pv, pv->current_sub);
             hb_buffer_close(&pv->current_sub);
         }
-        if (sub->s.start == sub->s.stop)
+        if (sub->s.flags & HB_BUF_FLAG_EOS)
         {
-            // Zero duration sub used to "clear" previous sub that had
+            // marker used to "clear" previous sub that had
             // an unknown duration
             hb_buffer_close(&sub);
         }
@@ -761,17 +866,14 @@ static void ApplyPGSSubs( hb_filter_private_t * pv, hb_buffer_t * buf )
         sub = hb_list_item( pv->sub_list, 0 );
         if ( sub->s.start <= buf->s.start )
         {
-            while ( sub )
-            {
-                ApplySub( pv, buf, sub );
-                sub = sub->sub;
-            }
+            hb_buffer_t *scaled = ScaleSubtitle(pv, sub, buf);
+            ApplySub( pv, buf, scaled );
+            hb_buffer_close(&scaled);
         }
     }
 }
 
-static int pgssub_init( hb_filter_object_t * filter,
-                        hb_filter_init_t * init )
+static int pgssub_post_init( hb_filter_object_t * filter, hb_job_t * job )
 {
     hb_filter_private_t * pv = filter->private_data;
 
@@ -804,7 +906,7 @@ static int pgssub_work( hb_filter_object_t * filter,
     hb_buffer_t * in = *buf_in;
     hb_buffer_t * sub;
 
-    if ( in->size <= 0 )
+    if (in->s.flags & HB_BUF_FLAG_EOF)
     {
         *buf_in = NULL;
         *buf_out = in;
@@ -815,6 +917,11 @@ static int pgssub_work( hb_filter_object_t * filter,
     // subtitle list
     while ( ( sub = hb_fifo_get( filter->subtitle->fifo_out ) ) )
     {
+        if (sub->s.flags & HB_BUF_FLAG_EOF)
+        {
+            hb_buffer_close(&sub);
+            break;
+        }
         hb_list_add( pv->sub_list, sub );
     }
 
@@ -826,22 +933,13 @@ static int pgssub_work( hb_filter_object_t * filter,
 }
 
 static int hb_rendersub_init( hb_filter_object_t * filter,
-                                 hb_filter_init_t * init )
+                              hb_filter_init_t * init )
 {
     filter->private_data = calloc( 1, sizeof(struct hb_filter_private_s) );
     hb_filter_private_t * pv = filter->private_data;
     hb_subtitle_t *subtitle;
     int ii;
 
-    if( filter->settings )
-    {
-        sscanf( filter->settings, "%d:%d:%d:%d",
-                &pv->crop[0],
-                &pv->crop[1],
-                &pv->crop[2],
-                &pv->crop[3]);
-    }
-
     // Find the subtitle we need
     for( ii = 0; ii < hb_list_count(init->job->list_subtitle); ii++ )
     {
@@ -859,30 +957,45 @@ static int hb_rendersub_init( hb_filter_object_t * filter,
         hb_log("rendersub: no subtitle marked for burn");
         return 1;
     }
+    return 0;
+}
+
+static int hb_rendersub_post_init( hb_filter_object_t * filter, hb_job_t *job )
+{
+    hb_filter_private_t * pv = filter->private_data;
+
+    pv->crop[0] = job->crop[0];
+    pv->crop[1] = job->crop[1];
+    pv->crop[2] = job->crop[2];
+    pv->crop[3] = job->crop[3];
 
     switch( pv->type )
     {
         case VOBSUB:
         {
-            return vobsub_init( filter, init );
+            return vobsub_post_init( filter, job );
         } break;
 
         case SSASUB:
         {
-            return ssa_init( filter, init );
+            return ssa_post_init( filter, job );
         } break;
 
         case SRTSUB:
-        case CC608SUB:
         case UTF8SUB:
         case TX3GSUB:
         {
-            return textsub_init( filter, init );
+            return textsub_post_init( filter, job );
+        } break;
+
+        case CC608SUB:
+        {
+            return cc608sub_post_init( filter, job );
         } break;
 
         case PGSSUB:
         {
-            return pgssub_init( filter, init );
+            return pgssub_post_init( filter, job );
         } break;
 
         default:
@@ -891,6 +1004,7 @@ static int hb_rendersub_init( hb_filter_object_t * filter,
             return 1;
         } break;
     }
+    return 0;
 }
 
 static int hb_rendersub_work( hb_filter_object_t * filter,
@@ -934,6 +1048,11 @@ static int hb_rendersub_work( hb_filter_object_t * filter,
 static void hb_rendersub_close( hb_filter_object_t * filter )
 {
     hb_filter_private_t * pv = filter->private_data;
+
+    if (pv->sws != NULL)
+    {
+        sws_freeContext(pv->sws);
+    }
     switch( pv->type )
     {
         case VOBSUB:
