@@ -1,6 +1,6 @@
 /* scan.c
 
-   Copyright (c) 2003-2015 HandBrake Team
+   Copyright (c) 2003-2017 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -29,34 +29,37 @@ typedef struct
     int            store_previews;
 
     uint64_t       min_title_duration;
-
 } hb_scan_t;
 
 #define PREVIEW_READ_THRESH (1024 * 1024 * 300)
 
 static void ScanFunc( void * );
 static int  DecodePreviews( hb_scan_t *, hb_title_t * title, int flush );
-static void LookForAudio( hb_title_t * title, hb_buffer_t * b );
+static void LookForAudio(hb_scan_t *scan, hb_title_t *title, hb_buffer_t *b);
 static int  AllAudioOK( hb_title_t * title );
 static void UpdateState1(hb_scan_t *scan, int title);
 static void UpdateState2(hb_scan_t *scan, int title);
 static void UpdateState3(hb_scan_t *scan, int preview);
 
-static const char *aspect_to_string( double aspect )
+static const char *aspect_to_string(hb_rational_t *dar)
 {
+    double aspect = (double)dar->num / dar->den;
     switch ( (int)(aspect * 9.) )
     {
         case 9 * 4 / 3:    return "4:3";
         case 9 * 16 / 9:   return "16:9";
     }
     static char arstr[32];
-    sprintf( arstr, aspect >= 1.? "%.2f:1" : "1:%.2f", aspect );
+    if (aspect >= 1)
+        sprintf(arstr, "%.2f:1", aspect);
+    else
+        sprintf(arstr, "1:%.2f", 1. / aspect );
     return arstr;
 }
 
 hb_thread_t * hb_scan_init( hb_handle_t * handle, volatile int * die,
-                            const char * path, int title_index, 
-                            hb_title_set_t * title_set, int preview_count, 
+                            const char * path, int title_index,
+                            hb_title_set_t * title_set, int preview_count,
                             int store_previews, uint64_t min_duration )
 {
     hb_scan_t * data = calloc( sizeof( hb_scan_t ), 1 );
@@ -70,7 +73,19 @@ hb_thread_t * hb_scan_init( hb_handle_t * handle, volatile int * die,
     data->preview_count  = preview_count;
     data->store_previews = store_previews;
     data->min_title_duration = min_duration;
-    
+
+    // Initialize scan state
+    hb_state_t state;
+#define p state.param.scanning
+    state.state   = HB_STATE_SCANNING;
+    p.title_cur   = 1;
+    p.title_count = 1;
+    p.preview_cur = 0;
+    p.preview_count = 1;
+    p.progress = 0.0;
+#undef p
+    hb_set_state(handle, &state);
+
     return hb_thread_init( "scan", ScanFunc, data, HB_NORMAL_PRIORITY );
 }
 
@@ -86,7 +101,7 @@ static void ScanFunc( void * _data )
     data->stream = NULL;
 
     /* Try to open the path as a DVD. If it fails, try as a file */
-    if( ( data->bd = hb_bd_init( data->path ) ) )
+    if( ( data->bd = hb_bd_init( data->h, data->path ) ) )
     {
         hb_log( "scan: BD has %d title(s)",
                 hb_bd_title_count( data->bd ) );
@@ -104,14 +119,14 @@ static void ScanFunc( void * _data )
             {
                 UpdateState1(data, i + 1);
                 hb_list_add( data->title_set->list_title,
-                             hb_bd_title_scan( data->bd, 
+                             hb_bd_title_scan( data->bd,
                              i + 1, data->min_title_duration ) );
             }
             feature = hb_bd_main_feature( data->bd,
                                           data->title_set->list_title );
         }
     }
-    else if( ( data->dvd = hb_dvd_init( data->path ) ) )
+    else if( ( data->dvd = hb_dvd_init( data->h, data->path ) ) )
     {
         hb_log( "scan: DVD has %d title(s)",
                 hb_dvd_title_count( data->dvd ) );
@@ -129,19 +144,19 @@ static void ScanFunc( void * _data )
             {
                 UpdateState1(data, i + 1);
                 hb_list_add( data->title_set->list_title,
-                             hb_dvd_title_scan( data->dvd, 
+                             hb_dvd_title_scan( data->dvd,
                             i + 1, data->min_title_duration ) );
             }
             feature = hb_dvd_main_feature( data->dvd,
                                            data->title_set->list_title );
         }
     }
-    else if ( ( data->batch = hb_batch_init( data->path ) ) )
+    else if ( ( data->batch = hb_batch_init( data->h, data->path ) ) )
     {
         if( data->title_index )
         {
             /* Scan this title only */
-            title = hb_batch_title_scan( data->batch, data->title_index );
+            title = hb_batch_title_scan(data->batch, data->title_index);
             if ( title )
             {
                 hb_list_add( data->title_set->list_title, title );
@@ -155,7 +170,7 @@ static void ScanFunc( void * _data )
                 hb_title_t * title;
 
                 UpdateState1(data, i + 1);
-                title = hb_batch_title_scan( data->batch, i + 1 );
+                title = hb_batch_title_scan(data->batch, i + 1);
                 if ( title != NULL )
                 {
                     hb_list_add( data->title_set->list_title, title );
@@ -165,9 +180,17 @@ static void ScanFunc( void * _data )
     }
     else
     {
-        data->title_index = 1;
+        // Title index 0 is not a valid title number and means scan all titles.
+        // So set title index to 1 in this scenario.
+        //
+        // Otherwise, set title index in new title to the index that was
+        // requested.  This preserves the original index created in batch
+        // mode.
+        if (data->title_index == 0)
+            data->title_index = 1;
         hb_title_t * title = hb_title_init( data->path, data->title_index );
-        if ( (data->stream = hb_stream_open( data->path, title, 1 ) ) != NULL )
+        data->stream = hb_stream_open(data->h, data->path, title, 1);
+        if (data->stream != NULL)
         {
             title = hb_stream_title_scan( data->stream, title );
             if ( title )
@@ -177,7 +200,7 @@ static void ScanFunc( void * _data )
         {
             hb_title_close( &title );
             hb_log( "scan: unrecognized file type" );
-            return;
+            goto finish;
         }
     }
 
@@ -217,6 +240,7 @@ static void ScanFunc( void * _data )
             hb_title_close( &title );
             continue;
         }
+        title->preview_count = npreviews;
 
         /* Make sure we found audio rates and bitrates */
         for( j = 0; j < hb_list_count( title->list_audio ); )
@@ -240,7 +264,7 @@ static void ScanFunc( void * _data )
 
         if ( data->dvd || data->bd )
         {
-            // The subtitle width and height needs to be set to the 
+            // The subtitle width and height needs to be set to the
             // title widht and height for DVDs.  title width and
             // height don't get set until we decode previews, so
             // we can't set subtitle width/height till we get here.
@@ -249,8 +273,8 @@ static void ScanFunc( void * _data )
                 hb_subtitle_t *subtitle = hb_list_item( title->list_subtitle, j );
                 if ( subtitle->source == VOBSUB || subtitle->source == PGSSUB )
                 {
-                    subtitle->width = title->width;
-                    subtitle->height = title->height;
+                    subtitle->width = title->geometry.width;
+                    subtitle->height = title->geometry.height;
                 }
             }
         }
@@ -264,9 +288,15 @@ static void ScanFunc( void * _data )
     {
         title      = hb_list_item( data->title_set->list_title, i );
         title->flags |= HBTF_SCAN_COMPLETE;
-#if defined(HB_TITLE_JOBS)
-        title->job = hb_job_init( title );
-#endif
+    }
+    if (hb_list_count(data->title_set->list_title) > 0)
+    {
+        strncpy(data->title_set->path, data->path, 1024);
+        data->title_set->path[1023] = 0;
+    }
+    else
+    {
+        data->title_set->path[0] = 0;
     }
 
 finish:
@@ -303,7 +333,7 @@ static inline int absdiff( int x, int y )
     return x < y ? y - x : x - y;
 }
 
-static inline int clampBlack( int x ) 
+static inline int clampBlack( int x )
 {
     // luma 'black' is 16 and anything less should be clamped at 16
     return x < 16 ? 16 : x;
@@ -460,11 +490,12 @@ static int has_resolution_change( info_list_t *info_list )
 
     if( !info_list[0].count )
         return 0;
-    w = info_list[0].info.width;
-    h = info_list[0].info.height;
+    w = info_list[0].info.geometry.width;
+    h = info_list[0].info.geometry.height;
     for ( i = 1; info_list[i].count; ++i )
     {
-        if ( w != info_list[i].info.width || h != info_list[i].info.height )
+        if (w != info_list[i].info.geometry.width ||
+            h != info_list[i].info.geometry.height)
             return 1;
     }
     return 0;
@@ -486,21 +517,25 @@ static int is_close_to( int val, int target, int thresh )
  **********************************************************************/
 static int DecodePreviews( hb_scan_t * data, hb_title_t * title, int flush )
 {
-    int             i, npreviews = 0, abort = 0;
-    hb_buffer_t   * buf, * buf_es;
-    hb_list_t     * list_es;
-    int progressive_count = 0;
-    int pulldown_count = 0;
-    int doubled_frame_count = 0;
-    int interlaced_preview_count = 0;
-    int frame_wait = 0;
-    int cc_wait = 10;
-    int frames;
-    hb_stream_t  * stream = NULL;
-    info_list_t * info_list = calloc( data->preview_count+1, sizeof(*info_list) );
+    int                i, npreviews = 0, abort = 0;
+    hb_buffer_t      * buf, * buf_es;
+    hb_buffer_list_t   list_es;
+    int                progressive_count = 0;
+    int                pulldown_count = 0;
+    int                doubled_frame_count = 0;
+    int                interlaced_preview_count = 0;
+    int                vid_samples = 0;
+    int                frame_wait = 0;
+    int                cc_wait = 10;
+    int                frames;
+    hb_stream_t      * stream = NULL;
+    info_list_t      * info_list;
+    int                abort_audio = 0;
+
+    info_list = calloc(data->preview_count+1, sizeof(*info_list));
     crop_record_t *crops = crop_record_init( data->preview_count );
 
-    list_es  = hb_list_init();
+    hb_buffer_list_clear(&list_es);
 
     if( data->batch )
     {
@@ -524,22 +559,34 @@ static int DecodePreviews( hb_scan_t * data, hb_title_t * title, int flush )
     }
     else if (data->batch)
     {
-        stream = hb_stream_open( title->path, title, 0 );
+        stream = hb_stream_open(data->h, title->path, title, 0);
     }
     else if (data->stream)
     {
-        stream = hb_stream_open( data->path, title, 0 );
+        stream = hb_stream_open(data->h, data->path, title, 0);
     }
 
     if (title->video_codec == WORK_NONE)
     {
         hb_error("No video decoder set!");
+        free(info_list);
+        crop_record_free(crops);
+        hb_stream_close(&stream);
         return 0;
     }
-    hb_work_object_t *vid_decoder = hb_get_work(title->video_codec);
+    hb_work_object_t *vid_decoder = hb_get_work(data->h, title->video_codec);
     vid_decoder->codec_param = title->video_codec_param;
     vid_decoder->title = title;
-    vid_decoder->init( vid_decoder, NULL );
+
+    if (vid_decoder->init(vid_decoder, NULL))
+    {
+        hb_error("Decoder init failed!");
+        free(info_list);
+        crop_record_free(crops);
+        free( vid_decoder );
+        hb_stream_close(&stream);
+        return 0;
+    }
 
     for( i = 0; i < data->preview_count; i++ )
     {
@@ -551,6 +598,9 @@ static int DecodePreviews( hb_scan_t * data, hb_title_t * title, int flush )
         {
             free( info_list );
             crop_record_free( crops );
+            vid_decoder->close( vid_decoder );
+            free( vid_decoder );
+            hb_stream_close(&stream);
             return 0;
         }
         if (data->bd)
@@ -623,46 +673,53 @@ static int DecodePreviews( hb_scan_t * data, hb_title_t * title, int flush )
 
         int total_read = 0, packets = 0;
         while (total_read < PREVIEW_READ_THRESH ||
-               (!AllAudioOK(title) && packets < 10000))
+              (!AllAudioOK(title) && packets < 10000))
         {
             if (data->bd)
             {
-              if( (buf = hb_bd_read( data->bd )) == NULL )
-              {
-                  if ( vid_buf )
-                  {
-                    break;
-                  }
-                  hb_log( "Warning: Could not read data for preview %d, skipped", i + 1 );
-                  abort = 1;
-                  goto skip_preview;
-              }
+                if( (buf = hb_bd_read( data->bd )) == NULL )
+                {
+                    // If we reach EOF and no audio, don't continue looking for
+                    // audio
+                    abort_audio = 1;
+                    if ( vid_buf )
+                    {
+                        break;
+                    }
+                    hb_log( "Warning: Could not read data for preview %d, skipped", i + 1 );
+                    // If we reach EOF and no video, don't continue looking for
+                    // video
+                    abort = 1;
+                    goto skip_preview;
+                }
             }
             else if (data->dvd)
             {
-              if( (buf = hb_dvd_read( data->dvd )) == NULL )
-              {
-                  if ( vid_buf )
-                  {
-                    break;
-                  }
-                  hb_log( "Warning: Could not read data for preview %d, skipped", i + 1 );
-                  abort = 1;
-                  goto skip_preview;
-              }
+                if( (buf = hb_dvd_read( data->dvd )) == NULL )
+                {
+                    abort_audio = 1;
+                    if ( vid_buf )
+                    {
+                        break;
+                    }
+                    hb_log( "Warning: Could not read data for preview %d, skipped", i + 1 );
+                    abort = 1;
+                    goto skip_preview;
+                }
             }
             else if (stream)
             {
-              if ( (buf = hb_stream_read(stream)) == NULL )
-              {
-                  if ( vid_buf )
-                  {
-                    break;
-                  }
-                  hb_log( "Warning: Could not read data for preview %d, skipped", i + 1 );
-                  abort = 1;
-                  goto skip_preview;
-              }
+                if ( (buf = hb_stream_read(stream)) == NULL )
+                {
+                    abort_audio = 1;
+                    if ( vid_buf )
+                    {
+                        break;
+                    }
+                    hb_log( "Warning: Could not read data for preview %d, skipped", i + 1 );
+                    abort = 1;
+                    goto skip_preview;
+                }
             }
             else
             {
@@ -682,11 +739,10 @@ static int DecodePreviews( hb_scan_t * data, hb_title_t * title, int flush )
             total_read += buf->size;
             packets++;
 
-            (hb_demux[title->demuxer])(buf, list_es, 0 );
+            (hb_demux[title->demuxer])(buf, &list_es, 0 );
 
-            while( ( buf_es = hb_list_item( list_es, 0 ) ) )
+            while ((buf_es = hb_buffer_list_rem_head(&list_es)) != NULL)
             {
-                hb_list_rem( list_es, buf_es );
                 if( buf_es->s.id == title->video_id && vid_buf == NULL )
                 {
                     vid_decoder->work( vid_decoder, &buf_es, &vid_buf );
@@ -700,6 +756,36 @@ static int DecodePreviews( hb_scan_t * data, hb_title_t * title, int flush )
                     //    additional frames to find the CCs.
                     if (vid_buf != NULL && (frame_wait || cc_wait))
                     {
+                        hb_work_info_t vid_info;
+                        if (vid_decoder->info(vid_decoder, &vid_info))
+                        {
+                            if (is_close_to(vid_info.rate.den, 900900, 100) &&
+                                (vid_buf->s.flags & PIC_FLAG_REPEAT_FIRST_FIELD))
+                            {
+                                /* Potentially soft telecine material */
+                                pulldown_count++;
+                            }
+
+                            if (vid_buf->s.flags & PIC_FLAG_REPEAT_FRAME)
+                            {
+                                // AVCHD-Lite specifies that all streams are
+                                // 50 or 60 fps.  To produce 25 or 30 fps, camera
+                                // makers are repeating all frames.
+                                doubled_frame_count++;
+                            }
+
+                            if (is_close_to(vid_info.rate.den, 1126125, 100 ))
+                            {
+                                // Frame FPS is 23.976 (meaning it's
+                                // progressive), so start keeping track of
+                                // how many are reporting at that speed. When
+                                // enough show up that way, we want to make
+                                // that the overall title FPS.
+                                progressive_count++;
+                            }
+                            vid_samples++;
+                        }
+
                         if (frames > 0 && vid_buf->s.frametype == HB_FRAME_I)
                             frame_wait = 0;
                         if (frame_wait || cc_wait)
@@ -711,18 +797,19 @@ static int DecodePreviews( hb_scan_t * data, hb_title_t * title, int flush )
                         frames++;
                     }
                 }
-                else if( ! AllAudioOK( title ) ) 
+                else if (!AllAudioOK(title) && !abort_audio)
                 {
-                    LookForAudio( title, buf_es );
+                    LookForAudio( data, title, buf_es );
                     buf_es = NULL;
                 }
                 if ( buf_es )
                     hb_buffer_close( &buf_es );
             }
 
-            if( vid_buf && AllAudioOK( title ) )
+            if (vid_buf && (abort_audio || AllAudioOK(title)))
                 break;
         }
+        hb_buffer_list_close(&list_es);
 
         if( ! vid_buf )
         {
@@ -747,44 +834,22 @@ static int DecodePreviews( hb_scan_t * data, hb_title_t * title, int flush )
             continue;
         }
 
+        if (vid_info.geometry.width  != vid_buf->f.width ||
+            vid_info.geometry.height != vid_buf->f.height)
+        {
+            hb_log( "scan: video geometry information does not match buffer" );
+            hb_buffer_close( &vid_buf );
+            continue;
+        }
         remember_info( info_list, &vid_info );
 
-        if( is_close_to( vid_info.rate_base, 900900, 100 ) &&
-            ( vid_buf->s.flags & PIC_FLAG_REPEAT_FIRST_FIELD ) )
-        {
-            /* Potentially soft telecine material */
-            pulldown_count++;
-        }
-
-        if( vid_buf->s.flags & PIC_FLAG_REPEAT_FRAME )
-        {
-            // AVCHD-Lite specifies that all streams are
-            // 50 or 60 fps.  To produce 25 or 30 fps, camera
-            // makers are repeating all frames.
-            doubled_frame_count++;
-        }
-
-        if( is_close_to( vid_info.rate_base, 1126125, 100 ) )
-        {
-            // Frame FPS is 23.976 (meaning it's progressive), so start keeping
-            // track of how many are reporting at that speed. When enough 
-            // show up that way, we want to make that the overall title FPS.
-            progressive_count++;
-        }
-
-        while( ( buf_es = hb_list_item( list_es, 0 ) ) )
-        {
-            hb_list_rem( list_es, buf_es );
-            hb_buffer_close( &buf_es );
-        }
-
         /* Check preview for interlacing artifacts */
         if( hb_detect_comb( vid_buf, 10, 30, 9, 10, 30, 9 ) )
         {
             hb_deep_log( 2, "Interlacing detected in preview frame %i", i+1);
             interlaced_preview_count++;
         }
-        
+
         if( data->store_previews )
         {
             hb_save_preview( data->h, title->index, i, vid_buf );
@@ -793,7 +858,7 @@ static int DecodePreviews( hb_scan_t * data, hb_title_t * title, int flush )
         /* Detect black borders */
 
         int top, bottom, left, right;
-        int h4 = vid_info.height / 4, w4 = vid_info.width / 4;
+        int h4 = vid_info.geometry.height / 4, w4 = vid_info.geometry.width / 4;
 
         // When widescreen content is matted to 16:9 or 4:3 there's sometimes
         // a thin border on the outer edge of the matte. On TV content it can be
@@ -803,7 +868,7 @@ static int DecodePreviews( hb_scan_t * data, hb_title_t * title, int flush )
         // we can crop the matte. The border width depends on the resolution
         // (12 pixels on 1080i looks visually the same as 4 pixels on 480i)
         // so we allow the border to be up to 1% of the frame height.
-        const int border = vid_info.height / 100;
+        const int border = vid_info.geometry.height / 100;
 
         for ( top = border; top < h4; ++top )
         {
@@ -826,14 +891,14 @@ static int DecodePreviews( hb_scan_t * data, hb_title_t * title, int flush )
         }
         for ( bottom = border; bottom < h4; ++bottom )
         {
-            if ( ! row_all_dark( vid_buf, vid_info.height - 1 - bottom ) )
+            if ( ! row_all_dark( vid_buf, vid_info.geometry.height - 1 - bottom ) )
                 break;
         }
         if ( bottom <= border )
         {
             for ( bottom = 0; bottom < border; ++bottom )
             {
-                if ( ! row_all_dark( vid_buf, vid_info.height - 1 - bottom ) )
+                if ( ! row_all_dark( vid_buf, vid_info.geometry.height - 1 - bottom ) )
                     break;
             }
             if ( bottom >= border )
@@ -848,7 +913,7 @@ static int DecodePreviews( hb_scan_t * data, hb_title_t * title, int flush )
         }
         for ( right = 0; right < w4; ++right )
         {
-            if ( ! column_all_dark( vid_buf, top, bottom, vid_info.width - 1 - right ) )
+            if ( ! column_all_dark( vid_buf, top, bottom, vid_info.geometry.width - 1 - right ) )
                 break;
         }
 
@@ -901,53 +966,53 @@ skip_preview:
         {
             title->video_codec_name = strdup( vid_info.name );
         }
-        title->width = vid_info.width;
-        title->height = vid_info.height;
-        if ( vid_info.rate && vid_info.rate_base )
+        title->geometry.width = vid_info.geometry.width;
+        title->geometry.height = vid_info.geometry.height;
+        if (vid_info.rate.num && vid_info.rate.den)
         {
-            // if the frame rate is very close to one of our "common" framerates,
-            // assume it actually is said frame rate; e.g. some 24000/1001 sources
-            // may have a rate_base of 1126124 (instead of 1126125)
+            // if the frame rate is very close to one of our "common"
+            // framerates, assume it actually is said frame rate;
+            // e.g. some 24000/1001 sources may have a rate.den of 1126124
+            // instead of 1126125
             const hb_rate_t *video_framerate = NULL;
             while ((video_framerate = hb_video_framerate_get_next(video_framerate)) != NULL)
             {
-                if (is_close_to(vid_info.rate_base, video_framerate->rate, 100))
+                if (is_close_to(vid_info.rate.den, video_framerate->rate, 100))
                 {
-                    vid_info.rate_base = video_framerate->rate;
+                    vid_info.rate.den = video_framerate->rate;
                     break;
                 }
             }
-            title->rate = vid_info.rate;
-            title->rate_base = vid_info.rate_base;
-            if( vid_info.rate_base == 900900 )
+            title->vrate = vid_info.rate;
+            if( vid_info.rate.den == 900900 )
             {
-                if( npreviews >= 4 && pulldown_count >= npreviews / 4 )
+                if (vid_samples >= 4 && pulldown_count >= vid_samples / 4)
                 {
-                    title->rate_base = 1126125;
+                    title->vrate.den = 1126125;
                     hb_deep_log( 2, "Pulldown detected, setting fps to 23.976" );
                 }
-                if( npreviews >= 2 && progressive_count >= npreviews / 2 )
+                if (vid_samples >= 2 && progressive_count >= vid_samples / 2)
                 {
                     // We've already deduced that the frame rate is 23.976,
                     // so set it back again.
-                    title->rate_base = 1126125;
+                    title->vrate.den = 1126125;
                     hb_deep_log( 2, "Title's mostly NTSC Film, setting fps to 23.976" );
                 }
             }
-            if( npreviews >= 2 && doubled_frame_count >= 3 * npreviews / 4 )
+            if (vid_samples >= 2 && doubled_frame_count >= 3 * vid_samples / 4)
             {
                 // We've detected that a significant number of the frames
                 // have been doubled in duration by repeat flags.
-                title->rate_base = 2 * vid_info.rate_base;
-                hb_deep_log( 2, "Repeat frames detected, setting fps to %.3f", (float)title->rate / title->rate_base );
+                title->vrate.den = 2 * vid_info.rate.den;
+                hb_deep_log(2, "Repeat frames detected, setting fps to %.3f",
+                            (float)title->vrate.num / title->vrate.den );
             }
         }
         title->video_bitrate = vid_info.bitrate;
 
-        if( vid_info.pixel_aspect_width && vid_info.pixel_aspect_height )
+        if( vid_info.geometry.par.num && vid_info.geometry.par.den )
         {
-            title->pixel_aspect_width = vid_info.pixel_aspect_width;
-            title->pixel_aspect_height = vid_info.pixel_aspect_height;
+            title->geometry.par = vid_info.geometry.par;
         }
         title->color_prim = vid_info.color_prim;
         title->color_transfer = vid_info.color_transfer;
@@ -956,13 +1021,16 @@ skip_preview:
         title->video_decode_support = vid_info.video_decode_support;
 
         // TODO: check video dimensions
-        title->opencl_support = !!hb_opencl_available();
-
-        // compute the aspect ratio based on the storage dimensions and the
-        // pixel aspect ratio (if supplied) or just storage dimensions if no PAR.
-        title->aspect = (double)title->width / (double)title->height;
-        title->aspect *= (double)title->pixel_aspect_width /
-                         (double)title->pixel_aspect_height;
+        hb_handle_t *hb_handle = (hb_handle_t *)data->h;
+        if (hb_get_opencl_enabled(hb_handle))
+        {
+             title->opencl_support = !!hb_opencl_available();
+        }
+        
+        // compute the aspect ratio based on the storage dimensions and PAR.
+        hb_reduce(&title->dar.num, &title->dar.den,
+                  title->geometry.par.num * title->geometry.width,
+                  title->geometry.height * title->geometry.par.den);
 
         // For unknown reasons some French PAL DVDs put the original
         // content's aspect ratio into the mpeg PAR even though it's
@@ -971,14 +1039,18 @@ skip_preview:
         // aspect ratio from the DVD metadata. So, if the aspect computed
         // from the PAR is different from the container's aspect we use
         // the container's aspect & recompute the PAR from it.
-        if( title->container_aspect && (int)(title->aspect * 9) != (int)(title->container_aspect * 9) )
+        if (data->dvd &&
+            (title->dar.num != title->container_dar.num ||
+             title->dar.den != title->container_dar.den))
         {
-            hb_log("scan: content PAR gives wrong aspect %.2f; "
-                   "using container aspect %.2f", title->aspect,
-                   title->container_aspect );
-            title->aspect = title->container_aspect;
-            hb_reduce( &title->pixel_aspect_width, &title->pixel_aspect_height,
-                       (int)(title->aspect * title->height + 0.5), title->width );
+            hb_log("scan: content PAR gives wrong aspect %d:%d; "
+                   "using container aspect %d:%d",
+                   title->dar.num, title->dar.den,
+                   title->container_dar.num, title->container_dar.den);
+            title->dar = title->container_dar;
+            hb_reduce(&title->geometry.par.num, &title->geometry.par.den,
+                      title->geometry.height * title->dar.num,
+                      title->geometry.width * title->dar.den);
         }
 
         // don't try to crop unless we got at least 3 previews
@@ -987,7 +1059,7 @@ skip_preview:
             sort_crops( crops );
             // The next line selects median cropping - at least
             // 50% of the frames will have their borders removed.
-            // Other possible choices are loose cropping (i = 0) where 
+            // Other possible choices are loose cropping (i = 0) where
             // no non-black pixels will be cropped from any frame and a
             // tight cropping (i = crops->n - (crops->n >> 2)) where at
             // least 75% of the frames will have their borders removed.
@@ -1000,11 +1072,18 @@ skip_preview:
 
         hb_log( "scan: %d previews, %dx%d, %.3f fps, autocrop = %d/%d/%d/%d, "
                 "aspect %s, PAR %d:%d",
-                npreviews, title->width, title->height, (float) title->rate /
-                (float) title->rate_base,
+                npreviews, title->geometry.width, title->geometry.height,
+                (float)title->vrate.num / title->vrate.den,
                 title->crop[0], title->crop[1], title->crop[2], title->crop[3],
-                aspect_to_string( title->aspect ), title->pixel_aspect_width,
-                title->pixel_aspect_height );
+                aspect_to_string(&title->dar),
+                title->geometry.par.num, title->geometry.par.den);
+
+        if (title->video_decode_support != HB_DECODE_SUPPORT_SW)
+        {
+            hb_log("scan: supported video decoders:%s%s",
+                   !(title->video_decode_support & HB_DECODE_SUPPORT_SW)    ? "" : " avcodec",
+                   !(title->video_decode_support & HB_DECODE_SUPPORT_QSV)   ? "" : " qsv");
+        }
 
         if( interlaced_preview_count >= ( npreviews / 2 ) )
         {
@@ -1020,12 +1099,8 @@ skip_preview:
     crop_record_free( crops );
     free( info_list );
 
-    while( ( buf_es = hb_list_item( list_es, 0 ) ) )
-    {
-        hb_list_rem( list_es, buf_es );
-        hb_buffer_close( &buf_es );
-    }
-    hb_list_close( &list_es );
+    hb_buffer_list_close(&list_es);
+
     if (data->bd)
       hb_bd_stop( data->bd );
     if (data->dvd)
@@ -1048,7 +1123,7 @@ skip_preview:
  * aren't (e.g., some European DVD Teletext streams use the same IDs as US ATSC
  * AC-3 audio).
  */
-static void LookForAudio( hb_title_t * title, hb_buffer_t * b )
+static void LookForAudio(hb_scan_t *scan, hb_title_t * title, hb_buffer_t * b)
 {
     int i;
 
@@ -1084,7 +1159,7 @@ static void LookForAudio( hb_title_t * title, hb_buffer_t * b )
     }
     hb_fifo_push( audio->priv.scan_cache, b );
 
-    hb_work_object_t *w = hb_codec_decoder( audio->config.in.codec );
+    hb_work_object_t *w = hb_audio_decoder(scan->h, audio->config.in.codec);
 
     if ( w == NULL || w->bsinfo == NULL )
     {
@@ -1115,7 +1190,8 @@ static void LookForAudio( hb_title_t * title, hb_buffer_t * b )
     hb_fifo_flush( audio->priv.scan_cache );
     hb_fifo_close( &audio->priv.scan_cache );
 
-    audio->config.in.samplerate = info.rate;
+    audio->config.in.samplerate = info.rate.num;
+    audio->config.in.sample_bit_depth = info.sample_bit_depth;
     audio->config.in.samples_per_frame = info.samples_per_frame;
     audio->config.in.bitrate = info.bitrate;
     audio->config.in.matrix_encoding = info.matrix_encoding;
@@ -1184,6 +1260,15 @@ static void LookForAudio( hb_title_t * title, hb_buffer_t * b )
         {
             switch (audio->config.in.codec)
             {
+                case HB_ACODEC_AC3:
+                    codec_name = "AC3";
+                    break;
+                case HB_ACODEC_FFEAC3:
+                    codec_name = "E-AC3";
+                    break;
+                case HB_ACODEC_FFTRUEHD:
+                    codec_name = "TrueHD";
+                    break;
                 case HB_ACODEC_DCA:
                     codec_name = "DTS";
                     break;
@@ -1193,6 +1278,9 @@ static void LookForAudio( hb_title_t * title, hb_buffer_t * b )
                 case HB_ACODEC_FFAAC:
                     codec_name = "AAC";
                     break;
+                case HB_ACODEC_FFFLAC:
+                    codec_name = "FLAC";
+                    break;
                 case HB_ACODEC_MP3:
                     codec_name = "MP3";
                     break;
@@ -1281,7 +1369,7 @@ static void LookForAudio( hb_title_t * title, hb_buffer_t * b )
     hb_log( "scan: audio 0x%x: %s, rate=%dHz, bitrate=%d %s", audio->id,
             info.name, audio->config.in.samplerate, audio->config.in.bitrate,
             audio->config.lang.description );
- 
+
     free( w );
     return;
 
